# python stuff
import logging
import os
import io
from time import sleep
import unittest
import json
from datetime import datetime
from requests import request

# our testing code starts here
# -----------------------------------------------------------------------------
from memberpress_client.member import Member  # noqa: E402
from memberpress_client.transaction import Transaction  # noqa: E402
from memberpress_client.subscription import Subscription  # noqa: E402
from memberpress_client.membership import Membership  # noqa: E402

logger = logging.getLogger(__name__)

# setup test data
HERE = os.path.abspath(os.path.dirname(__file__))


def load_test_member(test_file):
    with io.open(os.path.join(HERE, "data", "api", test_file), "rt", encoding="utf8") as f:
        return f.read()


# test data
# -----------------------------------------------------------------------------
valid_member_response = json.loads(load_test_member("valid-member.json"), strict=False)
invalid_member_missing_subscriptions = json.loads(
    load_test_member("invalid-member-missing-subscriptions.json"), strict=False
)
invalid_member_no_trx = json.loads(load_test_member("invalid-member-no-trx.json"), strict=False)


class TestMember(unittest.TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)

    def test_online_1(self):

        member = Member(username="jon")

        if not member.ready:
            self.assertEqual(member.should_raise_paywall, False)

        while not member.ready:
            sleep(1)

        registered_at = datetime.strptime("2022-10-13 22:27:16", "%Y-%m-%d %H:%M:%S")

        # class properties
        self.assertEqual(member.request, None)
        self.assertEqual(member.ready, True)
        self.assertEqual(type(member.member), dict)
        self.assertEqual(member.id, 10)
        self.assertEqual(member.email, "jon.spurling@crstrategypartners.com")
        self.assertEqual(member.username, "jon")
        self.assertEqual(member.nicename, "jon")
        self.assertEqual(member.url, None)
        self.assertEqual(member.message, "")
        self.assertEqual(member.registered_at, registered_at)
        self.assertEqual(member.first_name, "Jon")
        self.assertEqual(member.last_name, "Spurling")
        self.assertEqual(member.display_name, "Jon Spurling")
        self.assertEqual(member.active_txn_count, 0)
        self.assertEqual(member.expired_txn_count, 0)
        self.assertEqual(member.trial_txn_count, 0)
        self.assertGreater(member.login_count, 0)

        # dict structural integrity
        self.assertEqual(member.is_complete_dict, False)
        self.assertEqual(member.is_minimum_member_dict, True)
        self.assertEqual(member.is_valid, True)

        self.assertEqual(member.active_memberships, [])
        self.assertEqual(member.recent_subscriptions, [])
        self.assertEqual(member.recent_transactions, [])
        self.assertEqual(member.first_transaction, None)
        self.assertEqual(member.latest_transaction, None)

        if member.active_memberships is not None:
            self.assertEqual(len(member.active_memberships), 0)

        if member.recent_subscriptions is not None:
            self.assertEqual(len(member.recent_subscriptions), 0)

        if member.recent_transactions is not None:
            self.assertEqual(len(member.recent_transactions), 0)

        # advanced class properties - business rule support
        self.assertEqual(member.is_active_subscription, False)
        self.assertEqual(member.is_trial_subscription, False)
        self.assertEqual(member.should_raise_paywall, True)



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - url scaffolding
"""
from django.conf import settings
from django.urls import include, path


app_name = "memberpress_client"
urlpatterns = [
    # production, as a Django plugin running in a namespace
    path("api/v1/", include("memberpress_client.api.v1.urls")),
]

if settings.DEBUG:
    # only used for local development.
    from django.contrib import admin

    urlpatterns += [
        path("mp/api/v1/", include("memberpress_client.api.v1.urls")),
        path("admin/", admin.site.urls),
    ]



from unittest import TestCase
from unittest.mock import patch

from django.core.cache import cache

from memberpress_client.client import MemberpressAPIClient

class MockResponse:
    def __init__(self, json_data, status_code):
        self.json_data = json_data
        self.status_code = status_code

    def json(self):
        return self.json_data
    
    def raise_for_status(self):
        pass


class TestClient(TestCase):

    def setUp(self):
        cache.clear()

    @patch("memberpress_client.client.requests.get", return_value=MockResponse({"foo": "bar"}, 200))
    def test_response_caching(self, mock_get):
        client = MemberpressAPIClient()
        assert mock_get.call_count == 0
        # first call should make a request
        client.get("test")
        assert mock_get.call_count == 1
        # second call should retrieve from cache
        client.get("test")
        assert mock_get.call_count == 1
        # first call with params should make a request
        client.get("test", params={"param_1": "value_1"})
        assert mock_get.call_count == 2
        # second call with params should retrieve from cache
        client.get("test", params={"param_1": "value_1"})
        assert mock_get.call_count == 2
        # first call with multiple params should make a request
        client.get("test", params={"param_1": "value_1", "param_2": "value_2"})
        assert mock_get.call_count == 3
        # second call with params out of order should still retrieve from cache
        client.get("test", params={"param_2": "value_2", "param_1": "value_1"})
        assert mock_get.call_count == 3
        # third call with same params but different values should make a request
        client.get("test", params={"param_1": "value_1", "param_2": "value_3"})
        assert mock_get.call_count == 4
        # call again with same params but caching disabled should make a request
        client.get("test", params={"param_1": "value_1", "param_2": "value_3"}, enable_caching=False)
        assert mock_get.call_count == 5
        



# Python stuff
import json
import logging
from datetime import datetime

# our stuff
from memberpress_client.client import MemberpressAPIClient
from memberpress_client.constants import COMPLETE_TRANSACTION_DICT

logger = logging.getLogger(__name__)


class Transaction(MemberpressAPIClient):
    def __init__(self, transaction=None) -> None:
        super().__init__()
        self.init()
        if type(transaction) == dict:
            self.json = transaction
        else:
            logger.warning("received an invalid transaction object: {o}".format(o=transaction))

    def init(self):
        super().init()

    @property
    def is_complete_dict(self) -> bool:
        """
        validate that response is a json dict containing at least
        the keys in qc_keys. These are the dict keys returned by the
        MemberPress REST api "/me" endpoint for a subscribed user.
        """
        qc_keys = COMPLETE_TRANSACTION_DICT
        return self.is_valid_dict(self.json, qc_keys)

    @property
    def membership(self) -> int:
        return self.str2int(self.json["membership"])

    @property
    def member(self) -> int:
        return self.str2int(self.json["member"])

    @property
    def coupon(self) -> int:
        return self.str2int(self.json["coupon"])

    @property
    def subscription(self) -> int:
        return self.str2int(self.json["subscription"])

    @property
    def transaction(self) -> int:
        return self.str2int(self.json["transaction"])

    @property
    def id(self) -> int:
        return self.str2int(self.json["id"])

    @property
    def amount(self) -> float:
        return self.str2float(self.json["amount"])

    @property
    def total(self) -> float:
        return self.str2float(self.json["total"])

    @property
    def tax_amount(self) -> float:
        return self.str2float(self.json["tax_amount"])

    @property
    def tax_rate(self) -> float:
        return self.str2float(self.json["tax_rate"])

    @property
    def tax_desc(self) -> str:
        return self.json.get("tax_desc", None)

    @property
    def tax_class(self) -> str:
        return self.json.get("tax_class", None)

    @property
    def trans_num(self) -> str:
        return self.json.get("trans_num", None)

    @property
    def status(self) -> str:
        return self.json.get("status", None)

    @property
    def txn_type(self) -> str:
        return self.json.get("txn_type", None)

    @property
    def gateway(self) -> str:
        return self.json.get("gateway", None)

    @property
    def prorated(self) -> int:
        return self.str2int(self.json["prorated"])

    @property
    def created_at(self) -> datetime:
        return self.str2datetime(self.json.get("created_at"))

    @property
    def expires_at(self) -> datetime:
        return self.str2datetime(self.json.get("expires_at"))

    @property
    def corporate_account_id(self) -> int:
        return self.str2int(self.json["corporate_account_id"])

    @property
    def parent_transaction_id(self) -> int:
        return self.str2int(self.json["parent_transaction_id"])

    @property
    def tax_compound(self) -> int:
        return self.str2int(self.json["tax_compound"])

    @property
    def tax_shipping(self) -> int:
        return self.str2int(self.json["tax_shipping"])

    @property
    def response(self) -> str:
        return self.json.get("response", None)



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - utility and helper functions.
"""
# python stuff
import json
import logging
import pytz
from unittest.mock import MagicMock
from requests import Response

# django stuff
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist


# our  stuff

UTC = pytz.UTC
logger = logging.getLogger(__name__)

# to get past tests
try:
    User = get_user_model()
except Exception:
    User = None


def masked_dict(obj) -> dict:
    """
    To mask sensitive key / value in log entries.
    masks the value of specified key.

    obj: a dict or a string representation of a dict, or None

    example:
        2022-10-07 20:03:01,455 INFO memberpress_client.client.Client.register_user() request: path=/api/user/v1/account/registration/, data={
            "name": "__Pat_SelfReg-07",
            "username": "__Pat_SelfReg-07",
            "email": "pat.mcguire+Pat_SelfReg-07@cabinetoffice.gov.uk",
            "password": "*** -- REDACTED -- ***",
            "terms_of_service": true
        }

    """

    def redact(key: str, obj):
        if key in obj:
            obj[key] = "*** -- REDACTED -- ***"
        return obj

    obj = obj or {}
    obj = dict(obj)
    for key in settings.MEMBERPRESS_SENSITIVE_KEYS:
        obj = redact(key, obj)
    return obj


class MPJSONEncoder(json.JSONEncoder):
    """
    a custom encoder class.
    - smooth out bumps mostly related to test data.
    - ensure text returned is utf-8 encoded.
    - velvety smooth error handling, understanding that we mostly use
      this class for generating log data.
    """

    def default(self, obj):
        if isinstance(obj, bytes):
            return str(obj, encoding="utf-8")
        if isinstance(obj, MagicMock):
            return ""
        try:
            return json.JSONEncoder.default(self, obj)
        except Exception:
            # obj probably is not json serializable.
            return ""


def get_user(username):
    try:
        return User.objects.get(username=username)
    except ObjectDoesNotExist:
        logger.warning("could not find a User object for username {username}".format(username=username))


def log_trace(caller: str, path: str, data: dict) -> None:
    """
    add an application log entry for higher level defs that call the edxapp api.
    """
    logger.info(
        "memberpress_client.client.Client.{caller}() request: path={path}, data={data}".format(
            caller=caller, path=path, data=json.dumps(masked_dict(data), cls=MPJSONEncoder, indent=4)
        )
    )


def log_pretrip(caller: str, url: str, data: dict, operation: str = "") -> None:
    """
    add an application log entry immediately prior to calling the edxapp api.
    """
    logger.info(
        "memberpress_client.client.Client.{caller}() {operation}, request: url={url}, data={data}".format(
            operation=operation,
            caller=caller,
            url=url,
            data=json.dumps(masked_dict(data), cls=MPJSONEncoder, indent=4),
        )
    )


def log_postrip(caller: str, path: str, response: Response, operation: str = "") -> None:
    """
    log the api response immediately after calling the edxapp api.
    """
    status_code = response.status_code if response is not None else 599
    log_message = "memberpress_client.client.Client.{caller}() {operation}, response status_code={response_status_code}, path={path}".format(
        operation=operation, caller=caller, path=path, response_status_code=status_code
    )
    if 200 <= status_code <= 399:
        logger.info(log_message)
    else:
        try:
            response_content = (
                json.dumps(response.content, cls=MPJSONEncoder, indent=4)
                if response is not None
                else "No response object."
            )
            log_message += ", response_content={response_content}".format(response_content=response_content)
        except TypeError:
            # TypeError: cannot convert dictionary update sequence element #0 to a sequence
            # This happens occasionally. Appears to be a malformed dict in the response body.
            pass

        logger.error(log_message)



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - Member class
"""

# Python stuff
import logging
from datetime import datetime
import requests

# our stuff
from memberpress_client.client import MemberpressAPIClient
from memberpress_client.subscription import Subscription
from memberpress_client.transaction import Transaction
from memberpress_client.membership import Membership
from memberpress_client.constants import (
    MemberPressAPI_Endpoints,
    MemberPressAPI_Operations,
    COMPLETE_MEMBER_DICT,
    MINIMUM_MEMBER_DICT,
)

logger = logging.getLogger(__name__)


class Member(MemberpressAPIClient):
    """
    memberpress REST API client
    """

    _request = None
    _username = None

    _recent_subscriptions = None
    _recent_transactions = None
    _first_transaction = None
    _latest_transaction = None
    _active_memberships = None

    def __init__(self, username=None, user=None, request=None, response=None) -> None:
        """
        username <str>: a Wordpress username
        user <obj>: a Django user object
        request <requests> a Django requests.request object
        response <json> the json reponse from memberpress REST API
        """
        super().__init__()
        self.init()
        self.request = request
        self.json = response

        # 4th priority username
        if response:
            try:
                self._username = response["username"]
            except Exception:
                pass

        # 3rd priority username
        if request:
            try:
                self._username = request.user.username
            except Exception:
                pass

        # 2nd priority username
        if user:
            try:
                self._username = user.username
            except Exception:
                pass

        # 1st priority username
        if username:
            self._username = username

        self.validate()

        if self.username:
            # invoke the getter
            self.member

    def init(self):
        super().init()
        self._request = None
        self._username = None
        self._is_valid = False
        self._recent_subscriptions = None
        self._recent_transactions = None
        self._first_transaction = None
        self._latest_transaction = None
        self._active_memberships = None

    def validate(self) -> None:
        super().validate()

        def list_diff(list_1: list, list_2: list) -> list:
            diff = list(set(list_2) - set(list_1))
            return ",".join(diff)

        if not self.username:
            self._is_valid = False
            return

        if not self.member:
            logger.error("member property is not set for username {username}".format(username=self.username))
            self._is_valid = False

        if type(self.member) != dict:
            logger.error(
                "was expecting member object of type dict but received an object of type {t}".format(
                    t=type(self.member)
                )
            )
            self._is_valid = False

        if self.is_complete_dict:
            logger.debug("validated member response object for username {username}.".format(username=self.username))
            self._is_valid = True
            return

        if self.is_minimum_member_dict:
            self._is_valid = True
            return

        missing = list_diff(MINIMUM_MEMBER_DICT, self.member.keys())
        logger.warning(
            "member response object for username {username} is missing the following required keys: {missing}".format(
                username=self.username, missing=missing
            )
        )

    @property
    def request(self):
        return self._request

    @request.setter
    def request(self, value):
        if type(value) == requests.request or value is None:
            self.init()
            self._request = value
            self.validate()
        else:
            raise TypeError("Was expecting value of type request but received object of type {t}".format(t=type(value)))

    @property
    def member(self) -> dict:
        """
        implements a single-shot attempt at retreiving a member dict from the rest api.
        if all works as hoped then then set the property to the rest api dict result.

        Any errors or validation failures will result in self.member returning
        an empty dict {} for the life of the object instance.
        """
        # note: need to use private _username in order to avoid recursion.
        if self._username and not self.json and not self.locked:
            """
            expected result is a list containing 1 dict
            """
            path = MemberPressAPI_Endpoints.MEMBERPRESS_API_MEMBER_PATH(username=self._username)
            retval = self.get(path=path, operation=MemberPressAPI_Operations.GET_MEMBER)

            if type(retval) == list and len(retval) == 1:
                retval = retval[0]

            if type(retval) == list and len(retval) > 1:
                for d in retval:
                    if d.get("username") == self.username:
                        retval = d
                        break

            if type(retval) != dict:
                logger.warning("member() was expecting a return type of dict but received {t}.".format(t=type(retval)))
                retval = None

            self.json = retval
        # convert NoneType to a dict so that other class properties
        # can safely use the form, val = self.member.get("blah")
        return self.json or {}

    @property
    def id(self) -> int:
        return self.str2int(self.member.get("id"))

    @property
    def email(self) -> str:
        return self.str2email(self.member.get("email"))

    @property
    def username(self) -> str:
        if self._username:
            return self._username
        return self.member.get("username", None)

    @property
    def nicename(self) -> str:
        return self.member.get("nicename", None)

    @property
    def url(self) -> str:
        return self.str2url(self.member.get("url"))

    @property
    def message(self) -> str:
        return self.member.get("message", None)

    @property
    def registered_at(self) -> datetime:
        return self.str2datetime(self.member.get("registered_at"))

    @property
    def first_name(self) -> str:
        return self.member.get("first_name", None)

    @property
    def last_name(self) -> str:
        return self.member.get("last_name", None)

    @property
    def display_name(self) -> str:
        return self.member.get("display_name", None)

    @property
    def active_txn_count(self) -> int:
        """
        the number of historical financial transactions (ie Stripe, PayPal, etc.)
        that exist for this member.
        """
        return self.str2int(self.member.get("active_txn_count"))

    @property
    def expired_txn_count(self) -> int:
        """
        the number of historical financial transactions (ie Stripe, PayPal, etc.)
        that exist for this member with an expiration date in the past.
        """
        return self.str2int(self.member.get("expired_txn_count"))

    @property
    def trial_txn_count(self) -> int:
        """
        the number of free trials that exist for this member.
        """
        return self.str2int(self.member.get("trial_txn_count"))

    @property
    def login_count(self) -> int:
        """
        the number of times that this member has logged in to the
        Wordpress site hosting the memberpress REST API plugin.
        """
        return self.str2int(self.member.get("login_count"))

    @property
    def recent_subscriptions(self) -> list:
        if not self._recent_subscriptions and self.is_valid:
            recent_subscriptions = self.data.get("recent_subscriptions", [])
            self._recent_subscriptions = self.list_factory(recent_subscriptions, Subscription)
        return self._recent_subscriptions

    @property
    def recent_transactions(self) -> list:
        if not self._recent_transactions and self.is_valid:
            transactions = self.data.get("recent_transactions", [])
            self._recent_transactions = self.list_factory(transactions, Transaction)
        return self._recent_transactions

    @property
    def active_memberships(self) -> list:
        if not self._active_memberships and self.is_valid:
            memberships = self.data.get("active_memberships", [])
            self._active_memberships = self.list_factory(memberships, Membership)
        return self._active_memberships

    """
    ---------------------------------------------------------------------------
                                Computed properties
    ---------------------------------------------------------------------------
    """

    @property
    def is_complete_dict(self) -> bool:
        """
        validate that response is a json dict containing at least
        the keys in qc_keys. These are the dict keys returned by the
        MemberPress REST api "/me" endpoint for a subscribed user.
        """
        qc_keys = COMPLETE_MEMBER_DICT
        return self.is_valid_dict(self.member, qc_keys)

    @property
    def is_minimum_member_dict(self) -> bool:
        """
        validate that response is a json dict containing at least
        the minimum required keys in qc_keys. These are the dict keys
        containing information about the identity of the member and
        the status of the member's subscription.
        """
        qc_keys = MINIMUM_MEMBER_DICT
        return self.is_valid_dict(self.member, qc_keys)

    @property
    def first_transaction(self) -> Transaction:
        transaction_json = self.member.get("first_txn", None)
        if not self._first_transaction and transaction_json and self.is_valid:
            self._first_transaction = Transaction(transaction_json)
        return self._first_transaction

    @property
    def latest_transaction(self) -> Transaction:
        transaction_json = self.member.get("latest_txn", None)
        if not self._latest_transaction and transaction_json and self.is_valid:
            self._latest_transaction = Transaction(transaction_json)
        return self._latest_transaction

    @property
    def address(self) -> dict:
        return self.member.get("address", {}) if self.is_valid else {}

    @property
    def profile(self) -> dict:
        return self.member.get("profile", {}) if self.is_valid else {}

    """
    ---------------------------------------------------------------------------
                                Business Rule Implementations
    ---------------------------------------------------------------------------
    """

    @property
    def is_active_subscription(self) -> bool:
        if not self.is_valid:
            return False

        for subscription in self.recent_subscriptions:
            if subscription.status == "active":
                return True

        return False

    @property
    def is_trial_subscription(self) -> bool:
        if not self.is_valid:
            return False

        now = datetime.now()
        for membership in self.active_memberships:
            expire_date = membership.expire_fixed or now
            if expire_date >= now:
                return True

        return False

    @property
    def should_raise_paywall(self) -> bool:
        if self.is_active_subscription or self.is_trial_subscription:
            return False
        return self.ready



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

Local dev settings.
In order to mock up a realistic Django local dev environment i created a blank
project using:
    cookiecutter https://github.com/mabdullahadeel/cookiecutter-django-mysql
    pip install -r requirements/local.txt
    ./manage.py migrate
Afterwards i copied select chunks of the blank project into this repo.
"""
from pathlib import Path

import environ
import os

ROOT_DIR = Path(__file__).resolve(strict=True).parent.parent.parent
APPS_DIR = ROOT_DIR / "memberpress_client"
env = environ.Env()

READ_DOT_ENV_FILE = env.bool("DJANGO_READ_DOT_ENV_FILE", default=True)
if READ_DOT_ENV_FILE:
    ENV_PATH = os.path.join(APPS_DIR, "settings", ".envs", ".local", ".django")
    print("Loading environment variables from {ENV_PATH}".format(ENV_PATH=ENV_PATH))
    env.read_env(ENV_PATH)

# your local dev & test settings go here
# -----------------------------------------------------------------------------
MEMBERPRESS_API_KEY = env.str("MEMBERPRESS_API_KEY", "set-me-please")
MEMBERPRESS_API_BASE_URL = env.str("MEMBERPRESS_API_BASE_URL", "https://example.com/")
MEMBERPRESS_CACHE_EXPIRATION = env.int("MEMBERPRESS_CACHE_EXPIRATION", 60 * 60)
MEMBERPRESS_API_KEY_NAME = env.str("MEMBERPRESS_API_KEY_NAME", "MEMBERPRESS-API-KEY")
MEMBERPRESS_SENSITIVE_KEYS = env.list("MEMBERPRESS_SENSITIVE_KEYS") or [
    "password",
    "token",
    "client_id",
    "client_secret",
    "Authorization",
    "secret",
]

# -----------------------------------------------------------------------------
# Required to run ./manage.py runserver
# -----------------------------------------------------------------------------
DEBUG = True
LOGGING_CONFIG = None
LOGGING = None
FORCE_SCRIPT_NAME = None
ALLOWED_HOSTS = [
    "127.0.0.1",
    "0.0.0.0",
]
EMAIL_BACKEND = None
USE_I18N = False
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.filebased.FileBasedCache",
        "LOCATION": ".//django_cache",
    }
}
ROOT_URLCONF = "memberpress_client.urls"
SECRET_KEY = "shooby-doooby-do"


# GENERAL
# ------------------------------------------------------------------------------
# Local time zone. Choices are
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# though not all of them may be available with every OS.
# In Windows, this must be set to your system time zone.
TIME_ZONE = "UTC"
# https://docs.djangoproject.com/en/dev/ref/settings/#language-code
LANGUAGE_CODE = "en-us"
# https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1

# DATABASES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.mysql",
        "NAME": "memberpress_client",
        "USER": "mp_user",
        "PASSWORD": "mp",
        "HOST": "127.0.0.1",
        "PORT": "3306",
        "OPTIONS": {"init_command": "SET sql_mode='STRICT_TRANS_TABLES'"},
    }
}
DATABASES["default"]["ATOMIC_REQUESTS"] = True
# https://docs.djangoproject.com/en/stable/ref/settings/#std:setting-DEFAULT_AUTO_FIELD
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# URLS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#root-urlconf
ROOT_URLCONF = "memberpress_client.urls"
# https://docs.djangoproject.com/en/dev/ref/settings/#wsgi-application
WSGI_APPLICATION = "config.wsgi.application"

# APPS
# ------------------------------------------------------------------------------
DJANGO_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.humanize",  # Handy template tags
    "django.contrib.admin",
    "django.forms",
]
THIRD_PARTY_APPS = [
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "rest_framework",
    "rest_framework.authtoken",
    "corsheaders",
    "drf_spectacular",
    "django_extensions",
]

LOCAL_APPS = [
    "memberpress_client",
    # Your stuff: custom apps go here
]
# https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

# AUTHENTICATION
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#authentication-backends
# AUTHENTICATION_BACKENDS = []

# PASSWORDS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#password-hashers
# PASSWORD_HASHERS = []
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators
# AUTH_PASSWORD_VALIDATORS = []


# django-rest-framework
# -------------------------------------------------------------------------------
# django-rest-framework - https://www.django-rest-framework.org/api-guide/settings/
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": ("rest_framework.permissions.IsAuthenticated",),
}

# django-cors-headers - https://github.com/adamchainz/django-cors-headers#setup
CORS_URLS_REGEX = r"^/api/.*$"

# STATIC
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = str(ROOT_DIR / "staticfiles")
# https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = "/static/"
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = [str(APPS_DIR / "static")]
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

# MEDIA
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(APPS_DIR / "media")
# https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/"


# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # https://docs.djangoproject.com/en/dev/ref/settings/#dirs
        "DIRS": [str(APPS_DIR / "templates")],
        # https://docs.djangoproject.com/en/dev/ref/settings/#app-dirs
        "APP_DIRS": True,
        "OPTIONS": {
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]

# MIDDLEWARE
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#middleware
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.common.BrokenLinkEmailsMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

Listen for signals published by edxapp.
"""

# Python
import logging

# Django
from django.dispatch import receiver

# Open edX
try:
    from openedx.core.djangoapps.signals.signals import COURSE_GRADE_NOW_PASSED
    from lms.djangoapps.certificates.generation_handler import generate_certificate_task

    RUNNING_LOCALLY = False
except ImportError:
    from django.dispatch import Signal

    COURSE_GRADE_NOW_PASSED = Signal()
    RUNNING_LOCALLY = True


# this repo
from memberpress_client.member import Member

log = logging.getLogger(__name__)


@receiver(COURSE_GRADE_NOW_PASSED, dispatch_uid="stepwisemath_passing_learner")
def listen_for_passing_grade(sender, user, course_id, **kwargs):  # pylint: disable=unused-argument
    """
    Listen for a signal indicating that the user has passed a course run.
    """
    log.info(
        "Enrolled student {username} has achieved a passing grade in the course {course_id} [{kwargs}]".format(
            username=user.username, course_id=course_id, kwargs=kwargs
        )
    )

    member = Member(username=user.username)
    if RUNNING_LOCALLY:
        return
    if member.is_trial_subscription:
        log.info(
            "Cannot generate certificate because enrolled student {username} is on a trial membership.".format(
                username=user.username
            )
        )
        return
    if not member.is_active_subscription:
        log.info(
            "Cannot generate certificate because subscription for enrolled student {username} is inactive.".format(
                username=user.username
            )
        )
        return

    log.info("Creating certificate generation task for student {username}.".format(username=user.username))
    generate_certificate_task(user=user, course_key=course_id)



# python stuff
import os
import io
import unittest
import json
from datetime import datetime
from requests import request


# our testing code starts here
# -----------------------------------------------------------------------------
from memberpress_client.member import Member  # noqa: E402

# setup test data
HERE = os.path.abspath(os.path.dirname(__file__))


def load_test_member(test_file):
    with io.open(os.path.join(HERE, "data", "api", test_file), "rt", encoding="utf8") as f:
        return f.read()


# test data
# -----------------------------------------------------------------------------
valid_member_nosubscription = json.loads(load_test_member("valid-member-no-subscription.json"), strict=False)


class TestMember(unittest.TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)

    def test_offline_1(self):

        member = Member(request=None, response=valid_member_nosubscription)
        # registered_at = datetime.strptime("2022-10-07 22:21:58", "%Y-%m-%d %H:%M:%S")

        # class properties
        self.assertEqual(member.request, None)
        self.assertEqual(member.ready, True)
        self.assertEqual(type(member.member), dict)
        self.assertEqual(member.id, 10)
        self.assertEqual(member.email, "jon.spurling@crstrategypartners.com")
        self.assertEqual(member.username, "jon")
        self.assertEqual(member.nicename, "jon")
        self.assertEqual(member.url, None)
        self.assertEqual(member.message, "")
        # self.assertAlmostEqual(member.registered_at, registered_at)
        self.assertEqual(member.first_name, "Jon")
        self.assertEqual(member.last_name, "Spurling")
        self.assertEqual(member.display_name, "Jon Spurling")
        self.assertEqual(member.active_txn_count, 0)
        self.assertEqual(member.expired_txn_count, 0)
        self.assertEqual(member.trial_txn_count, 0)
        self.assertEqual(member.login_count, 0)

        # dict structural integrity
        self.assertEqual(member.is_complete_dict, False)
        self.assertEqual(member.is_minimum_member_dict, True)
        self.assertEqual(member.is_valid, True)

        self.assertEqual(type(member.active_memberships), list)
        self.assertEqual(type(member.recent_subscriptions), list)
        self.assertEqual(type(member.recent_transactions), list)
        self.assertEqual(member.first_transaction, None)
        self.assertEqual(member.latest_transaction, None)

        self.assertEqual(len(member.active_memberships), 0)
        self.assertEqual(len(member.recent_subscriptions), 0)
        self.assertEqual(len(member.recent_transactions), 0)

        # advanced class properties - business rule support
        self.assertEqual(member.is_active_subscription, False)
        self.assertEqual(member.is_trial_subscription, False)


if __name__ == "__main__":
    unittest.main()



# python stuff
import os
import io
import unittest
import json
from datetime import datetime
from requests import request


# our testing code starts here
# -----------------------------------------------------------------------------
from memberpress_client.member import Member  # noqa: E402
from memberpress_client.transaction import Transaction  # noqa: E402
from memberpress_client.subscription import Subscription  # noqa: E402
from memberpress_client.membership import Membership  # noqa: E402

# setup test data
HERE = os.path.abspath(os.path.dirname(__file__))


def load_test_member(test_file):
    with io.open(os.path.join(HERE, "data", "api", test_file), "rt", encoding="utf8") as f:
        return f.read()


# test data
# -----------------------------------------------------------------------------
valid_member_response = json.loads(load_test_member("valid-member.json"), strict=False)
invalid_member_missing_subscriptions = json.loads(
    load_test_member("invalid-member-missing-subscriptions.json"), strict=False
)
invalid_member_no_trx = json.loads(load_test_member("invalid-member-no-trx.json"), strict=False)


class TestMember(unittest.TestCase):
    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)

    def test_none_member_1(self):

        member = Member(request=None, response=None)  # noqa: F841

    def test_none_member_2(self):

        member = Member(request=None, response=None)
        # class properties
        self.assertEqual(member.request, None)
        self.assertEqual(member.ready, False)
        self.assertEqual(member.member, {})
        self.assertEqual(member.id, None)
        self.assertEqual(member.email, None)
        self.assertEqual(member.username, None)
        self.assertEqual(member.nicename, None)
        self.assertEqual(member.url, None)
        self.assertEqual(member.message, None)
        self.assertEqual(member.registered_at, None)
        self.assertEqual(member.first_name, None)
        self.assertEqual(member.last_name, None)
        self.assertEqual(member.display_name, None)
        self.assertEqual(member.active_txn_count, None)
        self.assertEqual(member.expired_txn_count, None)
        self.assertEqual(member.trial_txn_count, None)
        self.assertEqual(member.login_count, None)

        # dict structural integrity
        self.assertEqual(member.is_complete_dict, False)
        self.assertEqual(member.is_minimum_member_dict, False)
        self.assertEqual(member.is_valid, False)
        self.assertEqual(member.active_memberships, None)
        self.assertEqual(member.recent_subscriptions, None)
        self.assertEqual(member.recent_transactions, None)
        self.assertEqual(member.first_transaction, None)
        self.assertEqual(member.latest_transaction, None)
        self.assertEqual(member.address, {})
        self.assertEqual(member.profile, {})

        # advanced class properties - business rule support
        self.assertEqual(member.is_active_subscription, False)
        self.assertEqual(member.is_trial_subscription, False)

    def test_offline_1(self):

        member = Member(request=None, response=valid_member_response)
        registered_at = datetime.strptime("2022-10-07 22:21:58", "%Y-%m-%d %H:%M:%S")

        # class properties
        self.assertEqual(member.request, None)
        self.assertEqual(member.ready, True)
        self.assertEqual(type(member.member), dict)
        self.assertEqual(member.id, 8)
        self.assertEqual(member.email, "jon.spurling@crstrategypartners.com")
        self.assertEqual(member.username, "JonSpurling81")
        self.assertEqual(member.nicename, "jonspurling81")
        self.assertEqual(member.url, None)
        self.assertEqual(member.message, "")
        self.assertEqual(member.registered_at, registered_at)
        self.assertEqual(member.first_name, "Jon")
        self.assertEqual(member.last_name, "Spurling")
        self.assertEqual(member.display_name, "Jon Spurling")
        self.assertEqual(member.active_txn_count, 1)
        self.assertEqual(member.expired_txn_count, 0)
        self.assertEqual(member.trial_txn_count, 1)
        self.assertEqual(member.login_count, 1)

        # dict structural integrity
        self.assertEqual(member.is_complete_dict, True)
        self.assertEqual(member.is_minimum_member_dict, True)
        self.assertEqual(member.is_valid, True)

        self.assertEqual(type(member.active_memberships), list)
        self.assertEqual(type(member.recent_subscriptions), list)
        self.assertEqual(type(member.recent_transactions), list)
        self.assertEqual(type(member.first_transaction), Transaction)
        self.assertEqual(type(member.latest_transaction), Transaction)

        self.assertEqual(len(member.active_memberships), 1)
        self.assertEqual(len(member.recent_subscriptions), 1)
        self.assertEqual(len(member.recent_transactions), 1)

        # advanced class properties - business rule support
        self.assertEqual(member.is_active_subscription, True)
        self.assertEqual(member.is_trial_subscription, False)

    def test_offline_2(self):

        member = Member(request=None, response=invalid_member_missing_subscriptions)
        self.assertEqual(member.is_active_subscription, False)
        self.assertEqual(member.is_trial_subscription, False)

    def test_offline_3(self):

        member = Member(request=None, response=invalid_member_no_trx)

        self.assertEqual(member.is_complete_dict, False)
        self.assertEqual(member.is_minimum_member_dict, False)
        self.assertEqual(member.is_active_subscription, False)
        self.assertEqual(member.is_trial_subscription, False)

    def test_offline_4_first_transaction(self):

        member = Member(request=None, response=valid_member_response)
        trx = member.first_transaction

        self.assertEqual(trx.is_complete_dict, True)
        self.assertEqual(type(trx.json), dict)
        self.assertEqual(trx.membership, 2420)
        self.assertEqual(trx.member, 8)
        self.assertEqual(trx.coupon, 0)
        self.assertEqual(trx.subscription, 1)
        self.assertEqual(trx.id, 1)
        self.assertEqual(trx.amount, 0.00)
        self.assertEqual(trx.total, 0.00)
        self.assertEqual(trx.tax_amount, 0.00)
        self.assertEqual(trx.tax_rate, 0.000)
        self.assertEqual(trx.tax_desc, "")
        self.assertEqual(trx.tax_class, "standard")
        self.assertEqual(trx.trans_num, "sub_1LqOxIJ1UGflvSOWjTJpb6Sk")
        self.assertEqual(trx.status, "confirmed")
        self.assertEqual(trx.txn_type, "subscription_confirmation")
        self.assertEqual(trx.gateway, "rj1l52-6bw")
        self.assertEqual(trx.prorated, 0)
        self.assertEqual(trx.created_at, datetime.strptime("2022-10-07 22:21:58", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(trx.expires_at, datetime.strptime("2022-10-14 23:59:59", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(trx.corporate_account_id, 0)
        self.assertEqual(trx.parent_transaction_id, 0)
        self.assertEqual(trx.tax_compound, 0)
        self.assertEqual(trx.tax_shipping, 1)
        self.assertEqual(trx.response, None)
        self.assertEqual(trx.status, "confirmed")
        self.assertEqual(trx.member, 8)
        self.assertEqual(trx.prorated, 0)
        self.assertEqual(trx.gateway, "rj1l52-6bw")
        self.assertEqual(trx.parent_transaction_id, 0)

    def test_offline_5_recent_subscription(self):

        scr = Subscription(subscription=valid_member_response["recent_subscriptions"][0])
        self.assertEqual(scr.coupon, 0)
        self.assertEqual(scr.membership, 2420)
        self.assertEqual(scr.member, 8)
        self.assertEqual(scr.id, 1)
        self.assertEqual(scr.subscriber_id, "sub_1LqOxIJ1UGflvSOWjTJpb6Sk")
        self.assertEqual(scr.gateway, "rj1l52-6bw")
        self.assertEqual(scr.price, 9.00)
        self.assertEqual(scr.period, 1)
        self.assertEqual(scr.period_type, "months")
        self.assertEqual(scr.limit_cycles, 0)
        self.assertEqual(scr.limit_cycles_num, 2)
        self.assertEqual(scr.limit_cycles_action, "expire")
        self.assertEqual(scr.limit_cycles_expires_after, 1)
        self.assertEqual(scr.limit_cycles_expires_type, "days")
        self.assertEqual(scr.prorated_trial, 0)
        self.assertEqual(scr.trial, 1)
        self.assertEqual(scr.trial_days, 7)
        self.assertEqual(scr.trial_amount, None)
        self.assertEqual(scr.trial_tax_amount, 0.00)
        self.assertEqual(scr.trial_total, 0.00)
        self.assertEqual(scr.status, "active")
        self.assertEqual(scr.created_at, datetime.strptime("2022-10-07 22:23:48", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(scr.total, 9.00)
        self.assertEqual(scr.tax_rate, 0.000)
        self.assertEqual(scr.tax_amount, 0.00)
        self.assertEqual(scr.tax_desc, "")
        self.assertEqual(scr.tax_class, "standard")
        self.assertEqual(scr.cc_last4, 5673)
        self.assertEqual(scr.cc_exp_month, 12)
        self.assertEqual(scr.cc_exp_year, 2026)
        self.assertEqual(scr.token, "")
        self.assertEqual(scr.tax_compound, 0)
        self.assertEqual(scr.tax_shipping, 1)
        self.assertEqual(scr.response, None)

    def test_offline_6_membership(self):

        mbr = Membership(membership=valid_member_response["active_memberships"][0])
        self.assertEqual(mbr.id, 2420)
        self.assertEqual(mbr.title, "Student")
        self.assertEqual(mbr.excerpt, "")
        self.assertEqual(mbr.date, datetime.strptime("2022-10-04 03:37:31", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(mbr.status, "publish")
        self.assertEqual(mbr.author, 4)
        self.assertEqual(mbr.date_gmt, datetime.strptime("2022-10-04 03:37:31", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(mbr.modified, datetime.strptime("2022-10-12 16:10:02", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(mbr.modified_gmt, datetime.strptime("2022-10-12 16:10:02", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(mbr.group, 2438)
        self.assertEqual(mbr.price, 9.00)
        self.assertEqual(mbr.period, 1)
        self.assertEqual(mbr.period_type, "months")
        self.assertEqual(mbr.signup_button_text, "Sign Up")
        self.assertEqual(mbr.limit_cycles, False)
        self.assertEqual(mbr.limit_cycles_num, 2)
        self.assertEqual(mbr.limit_cycles_action, "expire")
        self.assertEqual(mbr.limit_cycles_expires_after, 1)
        self.assertEqual(mbr.limit_cycles_expires_type, "days")
        self.assertEqual(mbr.trial, 1)
        self.assertEqual(mbr.trial_days, 7)
        self.assertEqual(mbr.trial_amount, 0)
        self.assertEqual(mbr.trial_once, 1)
        self.assertEqual(mbr.group_order, 1)
        self.assertEqual(mbr.is_highlighted, 1)
        self.assertEqual(mbr.plan_code, "")
        self.assertEqual(mbr.pricing_title, "Student")
        self.assertEqual(mbr.pricing_show_price, True)
        self.assertEqual(mbr.pricing_display, "auto")
        self.assertEqual(mbr.custom_price, "")
        self.assertEqual(mbr.pricing_heading_txt, "Get the math help you need, every Step of the way.")
        self.assertEqual(mbr.pricing_footer_txt, "")
        self.assertEqual(mbr.pricing_button_txt, "Sign Up")
        self.assertEqual(mbr.pricing_button_position, "footer")
        self.assertEqual(type(mbr.pricing_benefits), list)
        self.assertEqual(mbr.register_price_action, "default")
        self.assertEqual(mbr.register_price, "")
        self.assertEqual(mbr.thank_you_page_enabled, False)
        self.assertEqual(mbr.thank_you_page_type, "message")
        self.assertEqual(mbr.thank_you_message, "")
        self.assertEqual(mbr.thank_you_page_id, 0)
        self.assertEqual(mbr.custom_login_urls_enabled, 1)
        self.assertEqual(mbr.custom_login_urls_default, "")
        self.assertEqual(type(mbr.custom_login_urls), list)
        self.assertEqual(mbr.expire_type, "none")
        self.assertEqual(mbr.expire_after, 1)
        self.assertEqual(mbr.expire_unit, "days")
        self.assertEqual(mbr.expire_fixed, datetime.strptime("2022-10-12", "%Y-%m-%d"))
        self.assertEqual(mbr.tax_exempt, False)
        self.assertEqual(mbr.tax_class, "standard")
        self.assertEqual(mbr.allow_renewal, False)
        self.assertEqual(mbr.access_url, "")
        self.assertEqual(mbr.disable_address_fields, False)
        self.assertEqual(mbr.simultaneous_subscriptions, False)
        self.assertEqual(mbr.use_custom_template, False)
        self.assertEqual(mbr.custom_template, "100-width.php")
        self.assertEqual(mbr.customize_payment_methods, 1)
        self.assertEqual(type(mbr.custom_payment_methods), list)
        self.assertEqual(mbr.customize_profile_fields, False)
        self.assertEqual(type(mbr.custom_profile_fields), list)
        self.assertEqual(mbr.cannot_purchase_message, "You don't have access to purchase this item.")


if __name__ == "__main__":
    unittest.main()



# Generated by Django 3.2.13 on 2022-10-18 12:39

from django.db import migrations, models
import django.utils.timezone
import jsonfield.fields
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="MemberpressEventLog",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "sender",
                    models.URLField(
                        blank=True, help_text="The site referrer. Example: https://wordpress-site.com/mb/webhooks/"
                    ),
                ),
                ("username", models.CharField(help_text="The username provided by memberpress.", max_length=50)),
                (
                    "event",
                    models.CharField(
                        choices=[
                            ("after-cc-expires-reminder", "after-cc-expires-reminder"),
                            ("after-member-signup-reminder", "after-member-signup-reminder"),
                            ("after-signup-abandoned-reminder", "after-signup-abandoned-reminder"),
                            ("after-sub-expires-reminder", "after-sub-expires-reminder"),
                            ("before-cc-expires-reminder", "before-cc-expires-reminder"),
                            ("before-sub-expires-reminder", "before-sub-expires-reminder"),
                            ("before-sub-renews-reminder", "before-sub-renews-reminder"),
                            ("before-sub-trial-ends", "before-sub-trial-ends"),
                            ("login", "login"),
                            ("member-account-updated", "member-account-updated"),
                            ("member-added", "member-added"),
                            ("member-deleted", "member-deleted"),
                            ("member-signup-completed", "member-signup-completed"),
                            ("mpca-course-completed", "mpca-course-completed"),
                            ("mpca-course-started", "mpca-course-started"),
                            ("mpca-lesson-completed", "mpca-lesson-completed"),
                            ("mpca-lesson-started", "mpca-lesson-started"),
                            ("mpca-quiz-attempt-completed", "mpca-quiz-attempt-completed"),
                            ("non-recurring-transaction-completed", "non-recurring-transaction-completed"),
                            ("non-recurring-transaction-expired", "non-recurring-transaction-expired"),
                            ("offline-payment-complete", "offline-payment-complete"),
                            ("offline-payment-pending", "offline-payment-pending"),
                            ("offline-payment-refunded", "offline-payment-refunded"),
                            ("recurring-transaction-completed", "recurring-transaction-completed"),
                            ("recurring-transaction-expired", "recurring-transaction-expired"),
                            ("recurring-transaction-failed", "recurring-transaction-failed"),
                            ("renewal-transaction-completed", "renewal-transaction-completed"),
                            ("sub-account-added", "sub-account-added"),
                            ("sub-account-removed", "sub-account-removed"),
                            ("subscription-created", "subscription-created"),
                            ("subscription-downgraded-to-one-time", "subscription-downgraded-to-one-time"),
                            ("subscription-downgraded-to-recurring", "subscription-downgraded-to-recurring"),
                            ("subscription-downgraded", "subscription-downgraded"),
                            ("subscription-expired", "subscription-expired"),
                            ("subscription-paused", "subscription-paused"),
                            ("subscription-resumed", "subscription-resumed"),
                            ("subscription-stopped", "subscription-stopped"),
                            ("subscription-upgraded-to-one-time", "subscription-upgraded-to-one-time"),
                            ("subscription-upgraded-to-recurring", "subscription-upgraded-to-recurring"),
                            ("subscription-upgraded", "subscription-upgraded"),
                            ("transaction-completed", "transaction-completed"),
                            ("transaction-expired", "transaction-expired"),
                            ("transaction-failed", "transaction-failed"),
                            ("transaction-refunded", "transaction-refunded"),
                            ("unidentified-event", "unidentified-event"),
                        ],
                        help_text="The memberpress event. Examples: recurring-transaction-completed",
                        max_length=50,
                    ),
                ),
                (
                    "event_type",
                    models.CharField(
                        choices=[
                            ("member", "member"),
                            ("membership", "membership"),
                            ("subscription", "subscription"),
                            ("transaction", "transaction"),
                        ],
                        help_text="The type of memberpress event: transaction, subscription, member, membership",
                        max_length=50,
                    ),
                ),
                (
                    "is_valid",
                    models.BooleanField(
                        help_text="True if the json received was validated by memberpress_client. False otherwise."
                    ),
                ),
                (
                    "is_processed",
                    models.BooleanField(
                        blank=True, default=False, help_text="True if this event has been analyzed and acted upon."
                    ),
                ),
                (
                    "json",
                    jsonfield.fields.JSONField(
                        blank=True, default={}, help_text="A json dict sent by the webhook event in the request body."
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "memberpress events",
            },
        ),
    ]



# Python stuff
import json
import logging
from datetime import datetime

# our stuff
from memberpress_client.client import MemberpressAPIClient
from memberpress_client.constants import COMPLETE_MEMBERSHIP_DICT

logger = logging.getLogger(__name__)


class Membership(MemberpressAPIClient):
    def __init__(self, membership=None) -> None:
        super().__init__()
        self.init()
        if type(membership) == dict:
            self.json = membership
        else:
            logger.warning("received an invalid membership object: {o}".format(o=membership))

    def init(self):
        super().init()

    @property
    def is_complete_dict(self) -> bool:
        """
        validate that response is a json dict containing at least
        the keys in qc_keys. These are the dict keys returned by the
        MemberPress REST api "/me" endpoint for a subscribed user.
        """
        qc_keys = COMPLETE_MEMBERSHIP_DICT
        return self.is_valid_dict(self.member, qc_keys)

    @property
    def id(self) -> int:
        try:
            return int(self.json["id"])
        except Exception:
            return None

    @property
    def title(self) -> str:
        return self.json.get("title")

    @property
    def content(self) -> str:
        return self.json.get("content")

    @property
    def excerpt(self) -> str:
        return self.json.get("excerpt")

    @property
    def date(self) -> datetime:
        return self.str2datetime(self.json.get("date", ""))

    @property
    def status(self) -> str:
        return self.json.get("status")

    @property
    def author(self) -> int:
        return self.str2int(self.json["author"])

    @property
    def date_gmt(self) -> datetime:
        return self.str2datetime(self.json.get("date_gmt"))

    @property
    def modified(self) -> datetime:
        return self.str2datetime(self.json.get("modified"))

    @property
    def modified_gmt(self) -> datetime:
        return self.str2datetime(self.json.get("modified_gmt", ""))

    @property
    def group(self) -> int:
        return self.str2int(self.json["group"])

    @property
    def price(self) -> float:
        return self.str2float(self.json["price"])

    @property
    def period(self) -> int:
        return self.str2int(self.json["period"])

    @property
    def period_type(self) -> str:
        return self.json.get("period_type")

    @property
    def signup_button_text(self) -> str:
        return self.json.get("signup_button_text")

    @property
    def limit_cycles(self) -> bool:
        return self.str2bool(self.json.get("limit_cycles"))

    @property
    def limit_cycles_num(self) -> int:
        return self.str2int(self.json["limit_cycles_num"])

    @property
    def limit_cycles_action(self) -> str:
        return self.json.get("limit_cycles_action")

    @property
    def limit_cycles_expires_after(self) -> int:
        return self.str2int(self.json["limit_cycles_expires_after"])

    @property
    def limit_cycles_expires_type(self) -> str:
        return self.json.get("limit_cycles_expires_type")

    @property
    def trial(self) -> int:
        return self.str2int(self.json["trial"])

    @property
    def trial_days(self) -> int:
        return self.str2int(self.json["trial_days"])

    @property
    def trial_amount(self) -> int:
        return self.str2int(self.json["trial_amount"])

    @property
    def trial_once(self) -> int:
        return self.str2int(self.json["trial_once"])

    @property
    def group_order(self) -> int:
        return self.str2int(self.json["group_order"])

    @property
    def is_highlighted(self) -> bool:
        retval = self.str2int(self.json["is_highlighted"])
        return True if retval == 1 else False

    @property
    def plan_code(self) -> str:
        return self.json.get("plan_code")

    @property
    def pricing_title(self) -> str:
        return self.json.get("pricing_title")

    @property
    def pricing_show_price(self) -> bool:
        return self.str2bool(self.json.get("pricing_show_price"))

    @property
    def pricing_display(self) -> str:
        return self.json.get("pricing_display")

    @property
    def custom_price(self) -> str:
        return self.json.get("custom_price")

    @property
    def pricing_heading_txt(self) -> str:
        return self.json.get("pricing_heading_txt")

    @property
    def pricing_footer_txt(self) -> str:
        return self.json.get("pricing_footer_txt")

    @property
    def pricing_button_txt(self) -> str:
        return self.json.get("pricing_button_txt")

    @property
    def pricing_button_position(self) -> str:
        return self.json.get("pricing_button_position")

    @property
    def pricing_benefits(self) -> list:
        return self.json.get("pricing_benefits")

    @property
    def register_price_action(self) -> str:
        return self.json.get("register_price_action")

    @property
    def register_price(self) -> str:
        return self.json.get("register_price")

    @property
    def thank_you_page_enabled(self) -> bool:
        return self.str2bool(self.json.get("thank_you_page_enabled"))

    @property
    def thank_you_page_type(self) -> str:
        return self.json.get("thank_you_page_type")

    @property
    def thank_you_message(self) -> str:
        return self.json.get("thank_you_message")

    @property
    def thank_you_page_id(self) -> int:
        return self.str2int(self.json["thank_you_page_id"])

    @property
    def custom_login_urls_enabled(self) -> int:
        return self.str2int(self.json["custom_login_urls_enabled"])

    @property
    def custom_login_urls_default(self) -> str:
        return self.json.get("custom_login_urls_default")

    @property
    def custom_login_urls(self) -> list:
        return self.json.get("custom_login_urls")

    @property
    def expire_type(self) -> str:
        return self.json.get("expire_type")

    @property
    def expire_after(self) -> int:
        return self.str2int(self.json["expire_after"])

    @property
    def expire_unit(self) -> str:
        return self.json.get("expire_unit")

    @property
    def expire_fixed(self) -> datetime:
        return self.str2datetime(self.json.get("expire_fixed"))

    @property
    def tax_exempt(self) -> bool:
        return self.str2bool(self.json.get("tax_exempt"))

    @property
    def tax_class(self) -> str:
        return self.json.get("tax_class")

    @property
    def allow_renewal(self) -> bool:
        return self.str2bool(self.json.get("allow_renewal"))

    @property
    def access_url(self) -> str:
        return self.json.get("access_url")

    @property
    def disable_address_fields(self) -> bool:
        return self.str2bool(self.json.get("disable_address_fields"))

    @property
    def simultaneous_subscriptions(self) -> bool:
        return self.str2bool(self.json.get("simultaneous_subscriptions"))

    @property
    def use_custom_template(self) -> bool:
        return self.str2bool(self.json.get("use_custom_template"))

    @property
    def custom_template(self) -> str:
        return self.json.get("custom_template")

    @property
    def customize_payment_methods(self) -> int:
        return self.str2int(self.json["customize_payment_methods"])

    @property
    def custom_payment_methods(self) -> list:
        return self.json.get("custom_payment_methods")

    @property
    def customize_profile_fields(self) -> bool:
        return self.str2bool(self.json.get("customize_profile_fields"))

    @property
    def custom_profile_fields(self) -> list:
        return self.json.get("custom_profile_fields")

    @property
    def cannot_purchase_message(self) -> str:
        return self.json.get("cannot_purchase_message")



"""
Common Pluggable Django App settings
"""


def plugin_settings(settings):
    """
    Injects local settings into django settings
    """

    settings.MEMBERPRESS_API_KEY = "set-me-please"  # noqa: F841
    settings.MEMBERPRESS_API_BASE_URL = "https://set-me-please.com/"  # noqa: F841



"""
base class
"""
from datetime import datetime
import logging
import validators

logger = logging.getLogger(__name__)


class Memberpress:
    _json = None  # the dict passed in the body of the webhook request object
    _is_valid = False  # set in validate()
    _locked = False
    _qc_keys = []  # set in __init__() of the child object. the data dict keys to validate
    _recent_subscriptions = None
    _recent_transactions = None
    _active_memberships = None

    def init(self):
        self._locked = False
        self._json = None
        self._qc_keys = []
        self._recent_subscriptions = None
        self._recent_transactions = None
        self._active_memberships = None
        logger.debug("initialized {t}".format(t=type(self)))

    def validate(self):
        if not self.json:
            self._is_valid = False
            return

        if not self.qc_keys:
            self._is_valid = False
            return

        # if everything passed then return True
        self._is_valid = True

    def str2bool(self, value):
        if type(value) == bool:
            return value

        if type(value) in [int, float]:
            value = int(value)
            return True if value == 1 else False

        if type(value) == str:
            return True if str(value).lower() == "true" else False

        return False

    def str2datetime(self, value):
        if type(value) != str:
            return None

        date_str = value[0:19]
        try:
            return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")
        except ValueError:
            pass

        try:
            return datetime.strptime(date_str, "%Y-%m-%d")
        except ValueError:
            pass

        logger.warning("Cannot convert datetime string {value}".format(value=value))

    def str2int(self, value):
        if type(value) == str and value == "":
            return None

        if type(value) in (str, int, float):
            try:
                return int(value)
            except ValueError:
                pass

    def str2float(self, value):
        if type(value) in (str, int, float):
            try:
                return float(value)
            except ValueError:
                pass

    def str2email(self, value):
        return value if validators.email(value) else None

    def str2url(self, value):
        try:
            return value if validators.url(value) else None
        except Exception:
            return None

    def is_valid_dict(self, response, qc_keys) -> bool:
        if not type(response) == dict:
            logger.warning(
                "is_valid_dict() was expecting a dict but received an object of type: {type}".format(
                    type=type(response)
                )
            )
            return False

        return all(key in response for key in qc_keys)

    def lock(self):
        self._locked = True

    def unlock(self):
        self._locked = False

    def list_factory(self, list_data: list, ListClass):
        retval = []
        for dict_json in list_data:
            obj = ListClass(dict_json)
            retval.append(obj)
        return retval

    @property
    def qc_keys(self):
        return self._qc_keys

    @qc_keys.setter
    def qc_keys(self, value):
        if type(value) == list:
            self._qc_keys = list(set(value))
        else:
            logger.warning("qc_keys.setter received an invalid value: {v}".format(v=value))

    @property
    def json(self) -> dict:
        return self._json or {}

    @json.setter
    def json(self, value):
        if type(value) == dict or value is None:
            self.init()
            self._json = value
        else:
            logger.warning("was expecting a value of type dict but receive type {t}".format(t=type(value)))

    @property
    def data(self) -> dict:
        # return the json["data"] dict if it exists.
        # otherwise revert to self.json
        retval = self.json.get("data")
        return retval if retval else self.json

    @property
    def is_valid(self) -> bool:
        return self._is_valid

    @property
    def locked(self) -> bool:
        return self._locked

    @property
    def ready(self) -> bool:
        return True if not self.locked and self.json and len(self.json) > 0 else False



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - events classes. A collection of
strongly-typed, higher-order classes, all descended from MemberpressEvent, that
consist of varying combinations of Member, Membership, Transaction,
and Subscription objects.

This is part of the webhooks implmentation. memberpress webhooks
post events to the "/mp/events" URL endpoint. the body of the request object
from memberpress contains a dict who's structure should match one of the
four dozen child classes of MemberpressEvent found in this module.

Usage:
from memberpress_client.events import get_event

memberpress_event = get_event(dict["event"])

Notes:
- **DO NOT** import MemberpressEvent nor its child classes.  Instead, use
  get_event(). This is intended to help avoid having to import dozens of child
  class into your module, not only for the sake of sanity but also better
  memory management.
"""
from datetime import datetime
import logging
from typing import TypeVar, Generic, Type

from memberpress_client.constants import (
    COMPLETE_TRANSACTION_DICT,
    COMPLETE_EVENT_DICT,
    PARTIAL_MEMBER_DICT,
    COMPLETE_FINANCIAL_SUBSCRIPTION_EVENT,
    COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT,
    MemberpressEvents,
    MemberpressEventTypes,
)
from memberpress_client.memberpress import Memberpress
from memberpress_client.member import Member
from memberpress_client.membership import Membership
from memberpress_client.transaction import Transaction
from memberpress_client.subscription import Subscription


logger = logging.getLogger(__name__)


MemberpressEventChild = TypeVar("MemberpressEventChild", bound="MemberpressEvent")


class MemberpressEvent(Generic[MemberpressEventChild], Memberpress):
    """
    Event base class
    """

    _event = None  # a string. example: "after-cc-expires-reminder"
    _event_type = None  # a string. example: "subscription"
    _member = None  # a member object
    _membership = None  # a membership object
    _transaction = None  # a transaction object
    _subscription = None  # a subscription object
    _is_valid = False  # set in validate()

    def __init__(self, data: dict) -> None:
        self.init()
        self.json = data

    @classmethod
    def factory(cls: Type[MemberpressEventChild], data: dict) -> MemberpressEventChild:
        return cls(data=data)

    def init(self):
        super().init()
        self._event = None
        self._event_type = None
        self._member = None
        self._membership = None
        self._transaction = None
        self._subscription = None
        self._is_valid = False

    def validate(self):
        """
        - parent class validations
        - integrity checks inside the event dict
        - structural integrity of the event dict
        """
        super().validate()
        if not self.is_valid:
            return

        if self.event != self.json.get("event", "missing event"):
            self._is_valid = False
            logger.warning("received a dict with inconsistent or missing 'event' key.")
            return

        if self.event_type != self.json.get("type", "missing event type"):
            self._is_valid = False
            logger.warning("received a dict with inconsistent or missing 'type' key.")
            return

        # this is the outer dict structure that all events share
        if not self.valid_base_keys:
            self._is_valid = False
            logger.warning("received a dict with no 'data' key.")
            return

        if not self.valid_event_keys:
            self._is_valid = False
            return

        self._is_valid = True

    @property
    def valid_base_keys(self):
        # all events have these three keys
        base_keys = ["event", "type", "data"]
        return self.is_valid_dict(self.json, qc_keys=base_keys)

    @property
    def valid_event_keys(self):
        # self.qc_keys is set in __init__() in the child classes
        # and contains the event-specific objects contained in
        # json["data"]
        if not self.qc_keys:
            return False
        return self.is_valid_dict(self.data, qc_keys=self.qc_keys)

    @property
    def has_member(self) -> bool:
        return MemberpressEventTypes.MEMBER in self.qc_keys

    @property
    def has_membership(self) -> bool:
        return MemberpressEventTypes.MEMBERSHIP in self.qc_keys

    @property
    def has_transaction(self) -> bool:
        return MemberpressEventTypes.TRANSACTION in self.qc_keys

    @property
    def has_subscription(self) -> bool:
        return MemberpressEventTypes.SUBSCRIPTION in self.qc_keys

    @property
    def event(self) -> str:
        return self._event

    @event.setter
    def event(self, value):
        if value is None:
            self._event = value
            return

        if type(value) == str:
            if value in MemberpressEvents.all():
                self._event = value
            else:
                logger.warning("event() received an unknown event type {value}".format(value=value))
        else:
            logger.warning("was expecting a value of type str but received type {t}".format(t=type(value)))

    @property
    def event_type(self) -> str:
        return self._event_type

    @event_type.setter
    def event_type(self, value):
        if value is None:
            self._event_type = value
            return

        if type(value) == str:
            if value in MemberpressEventTypes.all():
                self._event_type = value
            else:
                logger.warning("event_type() received an unknown event type {value}".format(value=value))
        else:
            logger.warning("was expecting a value of type str but received type {t}".format(t=type(value)))

    @property
    def membership(self) -> Membership:
        if self.has_membership and not self._membership:
            membership_dict = self.data.get(MemberpressEventTypes.MEMBERSHIP, {})
            self._membership = Membership(membership=membership_dict)
        return self._membership

    @property
    def member(self) -> Member:
        if self.has_member and not self._member:
            # note that the member dict resides in either of two locations
            # depending on the kind of event we received.
            member_dict = (
                self.data
                if self.event_type == MemberpressEventTypes.MEMBER
                else self.data.get(MemberpressEventTypes.MEMBER, {})
            )
            self._member = Member(response=member_dict)
        return self._member

    @property
    def transaction(self) -> Transaction:
        if self.has_transaction and not self._transaction:
            # note that the transaction object resides inside the body of the
            # of the data dict.
            transaction_dict = self.data
            self._transaction = Transaction(transaction=transaction_dict)
        return self._transaction

    @property
    def subscription(self) -> Subscription:
        if self.has_subscription and not self._subscription:
            subscription_dict = self.data.get(MemberpressEventTypes.SUBSCRIPTION, {})
            self._subscription = Subscription(subscription=subscription_dict)
        return self._subscription

    @property
    def recent_subscriptions(self) -> list:
        if not self._recent_subscriptions and self.is_valid:
            recent_subscriptions = self.data.get("recent_subscriptions", [])
            self._recent_subscriptions = self.list_factory(recent_subscriptions, Subscription)
        return self._recent_subscriptions

    @property
    def recent_transactions(self) -> list:
        if not self._recent_transactions and self.is_valid:
            transactions = self.data.get("recent_transactions", [])
            self._recent_transactions = self.list_factory(transactions, Transaction)
        return self._recent_transactions

    @property
    def active_memberships(self) -> list:
        if not self._active_memberships and self.is_valid:
            memberships = self.data.get("active_memberships", [])
            self._active_memberships = self.list_factory(memberships, Membership)
        return self._active_memberships

    # -------------------------------------------------------------------------
    # Event attributes
    # -------------------------------------------------------------------------

    @property
    def active_txn_count(self) -> int:
        return self.str2int(self.data.get("active_txn_count"))

    @property
    def address(self) -> dict:
        return self.data.get("address", {})

    @property
    def amount(self) -> float:
        return self.str2float(self.data.get("amount", 0.00))

    @property
    def cc_exp_month(self) -> int:
        return self.str2int(self.data.get("cc_exp_month"))

    @property
    def cc_exp_year(self) -> int:
        return self.str2int(self.data.get("cc_exp_year"))

    @property
    def cc_last4(self) -> int:
        return self.str2int(self.data.get("cc_last4"))

    @property
    def corporate_account_id(self) -> int:
        return self.str2int(self.data.get("corporate_account_id"))

    @property
    def coupon(self) -> int:
        return self.str2int(self.data.get("coupon"))

    @property
    def created_at(self) -> datetime:
        return self.str2datetime(self.data.get("created_at"))

    @property
    def display_name(self) -> str:
        return self.data.get("display_name", "")

    @property
    def email(self) -> str:
        return self.str2email(self.data.get("email"))

    @property
    def expired_txn_count(self) -> int:
        return self.str2int(self.data.get("expired_txn_count"))

    @property
    def expires_at(self) -> datetime:
        return self.str2datetime(self.data.get("expires_at"))

    @property
    def first_name(self) -> str:
        return self.data.get("first_name")

    @property
    def gateway(self) -> str:
        return self.data.get("gateway")

    @property
    def id(self) -> int:
        return self.str2int(self.data.get("id"))

    @property
    def last_name(self) -> str:
        return self.data.get("last_name")

    @property
    def limit_cycles(self) -> bool:
        return self.str2bool(self.data.get("limit_cycles", "false"))

    @property
    def limit_cycles_action(self) -> str:
        return self.data.get("limit_cycles_action")

    @property
    def limit_cycles_expires_after(self) -> int:
        return self.str2int(self.data.get("limit_cycles_expires_after"))

    @property
    def limit_cycles_expires_type(self) -> str:
        return self.data.get("limit_cycles_expires_type")

    @property
    def limit_cycles_num(self) -> int:
        return self.str2int(self.data.get("limit_cycles_num"))

    @property
    def login_count(self) -> int:
        return self.str2int(self.data.get("login_count"))

    @property
    def message(self) -> str:
        return self.data.get("message")

    @property
    def nicename(self) -> str:
        return self.data.get("nicename")

    @property
    def parent_transaction_id(self) -> int:
        return self.str2int(self.data.get("parent_transaction_id"))

    @property
    def period(self) -> int:
        return self.str2int(self.data.get("period"))

    @property
    def period_type(self) -> str:
        return self.data.get("period_type")

    @property
    def price(self) -> float:
        return self.str2float(self.data.get("price"))

    @property
    def profile(self) -> dict:
        return self.data.get("profile", {})

    @property
    def prorated(self) -> int:
        return self.str2int(self.data.get("prorated"))

    @property
    def prorated_trial(self) -> int:
        return self.str2int(self.data.get("prorated_trial"))

    @property
    def rebill(self) -> bool:
        return self.str2bool(self.json.get("rebill"))

    @property
    def registered_at(self) -> datetime:
        return self.str2datetime(self.data.get("registered_at")) if self.username else None

    @property
    def response(self):
        return self.data.get("response")

    @property
    def status(self) -> str:
        return self.data.get("status")

    @property
    def sub_count(self) -> int:
        return self.str2int(self.data.get("sub_count"))

    @property
    def subscr_id(self) -> str:
        return self.data.get("subscr_id")

    @property
    def subscription_payment_index(self) -> bool:
        return self.str2bool(self.json.get("subscription_payment_index"))

    @property
    def tax_amount(self) -> float:
        return self.str2float(self.data.get("tax_amount"))

    @property
    def tax_class(self) -> str:
        return self.data.get("tax_class")

    @property
    def tax_compound(self) -> int:
        return self.str2int(self.data.get("tax_compound"))

    @property
    def tax_desc(self) -> str:
        return self.data.get("tax_desc")

    @property
    def tax_rate(self) -> float:
        return self.str2float(self.data.get("tax_rate"))

    @property
    def tax_shipping(self) -> int:
        return self.str2int(self.data.get("tax_shipping"))

    @property
    def token(self) -> str:
        return self.data.get("token")

    @property
    def total(self) -> float:
        return self.str2float(self.data.get("total"))

    @property
    def trans_num(self) -> str:
        return self.data.get("trans_num")

    @property
    def trial(self) -> int:
        return self.str2int(self.data.get("trial"))

    @property
    def trial_amount(self) -> float:
        return self.str2float(self.data.get("trial_amount"))

    @property
    def trial_days(self) -> int:
        return self.str2int(self.data.get("trial_days"))

    @property
    def trial_tax_amount(self) -> float:
        return self.str2float(self.data.get("trial_tax_amount"))

    @property
    def trial_total(self) -> float:
        return self.str2float(self.data.get("trial_total"))

    @property
    def trial_txn_count(self) -> int:
        return self.str2int(self.data.get("trial_txn_count"))

    @property
    def txn_type(self) -> str:
        return self.data.get("txn_type")

    @property
    def url(self) -> str:
        return self.str2url(self.data.get("url"))

    @property
    def username(self) -> str:
        return self.data.get("username")


class MEAfterCCExpiresReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.AFTER_CC_EXPIRES_REMINDER
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_EVENT_DICT
        self.validate()


class MEAfterMemberSignupReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.AFTER_MEMBER_SIGNUP_REMINDER
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEAfterSignupAbandonedReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.AFTER_SIGNUP_ABANDONED_REMINDER
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEAfterSubExpiresReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.AFTER_SUB_EXPIRES_REMINDER
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEBeforeCCExpiresReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.BEFORE_CC_EXPIRES_REMINDER
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [MemberpressEventTypes.MEMBERSHIP, MemberpressEventTypes.MEMBER] + COMPLETE_EVENT_DICT
        self.validate()


class MEBeforeSubExpiresReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.BEFORE_SUB_EXPIRES_REMINDER
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEBeforeSubRenewsReminder(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.BEFORE_SUB_RENEWS_REMINDER
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEBeforeSubTrialEnds(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.BEFORE_SUB_TRIAL_ENDS
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [MemberpressEventTypes.MEMBERSHIP, MemberpressEventTypes.MEMBER] + COMPLETE_EVENT_DICT
        self.validate()


class MELogin(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.LOGIN
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMemberAccountUpdated(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MEMBER_ACCOUNT_UPDATED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMemberAdded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MEMBER_ADDED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMemberDeleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MEMBER_DELETED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMemberSignupCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MEMBER_SIGNUP_COMPLETED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMPCACourseCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MPCA_COURSE_COMPLETED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMPCACourseStarted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MPCA_COURSE_STARTED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMPCALessonCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MPCA_LESSON_COMPLETED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMPCALessonStarted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MPCA_LESSON_STARTED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MEMPCALQuizAttemptCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.MPCA_QUIZ_ATTEMPT_COMPLETED
        self.event_type = MemberpressEventTypes.MEMBER
        self.qc_keys = [] + PARTIAL_MEMBER_DICT
        self.validate()


class MENonRecurringTransactionCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.NON_RECURRING_TRANSACTION_COMPLETED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MENonRecurringTransactionExpired(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.NON_RECURRING_TRANSACTION_EXPIRED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEOfflinePaymentComplete(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.OFFLINE_PAYMENT_COMPLETE
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEOfflinePaymentPending(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.OFFLINE_PAYMENT_PENDING
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEOfflinePaymentRefunded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.OFFLINE_PAYMENT_REFUNDED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MERecurringTransactionCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.RECURRING_TRANSACTION_COMPLETED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MERecurringTransactionExpired(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.RECURRING_TRANSACTION_EXPIRED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MERecurringTransactionFailed(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.RECURRING_TRANSACTION_FAILED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MERenewalTransactionCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.RENEWAL_TRANSACTION_COMPLETED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MESubAccountAdded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUB_ACCOUNT_ADDED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MESubAccountRemoved(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUB_ACCOUNT_REMOVED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MESubscriptionCreated(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_CREATED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [MemberpressEventTypes.MEMBERSHIP, MemberpressEventTypes.MEMBER] + COMPLETE_EVENT_DICT
        self.validate()


class MESubscriptionDowngradedToOneTime(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_ONE_TIME
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_FINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionDowngradedToRecurring(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_RECURRING
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [MemberpressEventTypes.MEMBERSHIP, MemberpressEventTypes.MEMBER] + COMPLETE_EVENT_DICT
        self.validate()


class MESubscriptionDowngraded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_DOWNGRADED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionExpired(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_EXPIRED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionPaused(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_PAUSED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionResumed(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_RESUMED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionStopped(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_STOPPED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionUpgradedToOneTime(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_ONE_TIME
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MESubscriptionUpgradedToRecurring(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_RECURRING
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class MESubscriptionUpgraded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.SUBSCRIPTION_UPGRADED
        self.event_type = MemberpressEventTypes.SUBSCRIPTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
        ] + COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT
        self.validate()


class METransactionCompleted(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.TRANSACTION_COMPLETED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class METransactionExpired(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.TRANSACTION_EXPIRED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class METransactionFailed(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.TRANSACTION_FAILED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class METransactionRefunded(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.TRANSACTION_REFUNDED
        self.event_type = MemberpressEventTypes.TRANSACTION
        self.qc_keys = [
            MemberpressEventTypes.MEMBERSHIP,
            MemberpressEventTypes.MEMBER,
            MemberpressEventTypes.SUBSCRIPTION,
        ] + COMPLETE_TRANSACTION_DICT
        self.validate()


class MEUnidentifiedEvent(MemberpressEvent):
    def __init__(self, data: dict) -> None:
        super().__init__(data)
        self.event = MemberpressEvents.UNIDENTIFIED_EVENT
        self.event_type = ""
        self.qc_keys = []
        self.validate()


MEMBERPRESS_EVENT_CLASSES = {
    MemberpressEvents.AFTER_CC_EXPIRES_REMINDER: MEAfterCCExpiresReminder,
    MemberpressEvents.AFTER_MEMBER_SIGNUP_REMINDER: MEAfterMemberSignupReminder,
    MemberpressEvents.AFTER_SIGNUP_ABANDONED_REMINDER: MEAfterSignupAbandonedReminder,
    MemberpressEvents.AFTER_SUB_EXPIRES_REMINDER: MEAfterSubExpiresReminder,
    MemberpressEvents.BEFORE_CC_EXPIRES_REMINDER: MEBeforeCCExpiresReminder,
    MemberpressEvents.BEFORE_SUB_EXPIRES_REMINDER: MEBeforeSubExpiresReminder,
    MemberpressEvents.BEFORE_SUB_RENEWS_REMINDER: MEBeforeSubRenewsReminder,
    MemberpressEvents.BEFORE_SUB_TRIAL_ENDS: MEBeforeSubTrialEnds,
    MemberpressEvents.LOGIN: MELogin,
    MemberpressEvents.MEMBER_ACCOUNT_UPDATED: MEMemberAccountUpdated,
    MemberpressEvents.MEMBER_ADDED: MEMemberAdded,
    MemberpressEvents.MEMBER_DELETED: MEMemberDeleted,
    MemberpressEvents.MEMBER_SIGNUP_COMPLETED: MEMemberSignupCompleted,
    MemberpressEvents.MPCA_COURSE_COMPLETED: MEMPCACourseCompleted,
    MemberpressEvents.MPCA_COURSE_STARTED: MEMPCACourseStarted,
    MemberpressEvents.MPCA_LESSON_COMPLETED: MEMPCALessonCompleted,
    MemberpressEvents.MPCA_LESSON_STARTED: MEMPCALessonStarted,
    MemberpressEvents.MPCA_QUIZ_ATTEMPT_COMPLETED: MEMPCALQuizAttemptCompleted,
    MemberpressEvents.NON_RECURRING_TRANSACTION_COMPLETED: MENonRecurringTransactionCompleted,
    MemberpressEvents.NON_RECURRING_TRANSACTION_EXPIRED: MENonRecurringTransactionExpired,
    MemberpressEvents.OFFLINE_PAYMENT_COMPLETE: MEOfflinePaymentComplete,
    MemberpressEvents.OFFLINE_PAYMENT_PENDING: MEOfflinePaymentPending,
    MemberpressEvents.OFFLINE_PAYMENT_REFUNDED: MEOfflinePaymentRefunded,
    MemberpressEvents.RECURRING_TRANSACTION_COMPLETED: MERecurringTransactionCompleted,
    MemberpressEvents.RECURRING_TRANSACTION_EXPIRED: MERecurringTransactionExpired,
    MemberpressEvents.RECURRING_TRANSACTION_FAILED: MERecurringTransactionFailed,
    MemberpressEvents.RENEWAL_TRANSACTION_COMPLETED: MERenewalTransactionCompleted,
    MemberpressEvents.SUB_ACCOUNT_ADDED: MESubAccountAdded,
    MemberpressEvents.SUB_ACCOUNT_REMOVED: MESubAccountRemoved,
    MemberpressEvents.SUBSCRIPTION_CREATED: MESubscriptionCreated,
    MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_ONE_TIME: MESubscriptionDowngradedToOneTime,
    MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_RECURRING: MESubscriptionDowngradedToRecurring,
    MemberpressEvents.SUBSCRIPTION_DOWNGRADED: MESubscriptionDowngraded,
    MemberpressEvents.SUBSCRIPTION_EXPIRED: MESubscriptionExpired,
    MemberpressEvents.SUBSCRIPTION_PAUSED: MESubscriptionPaused,
    MemberpressEvents.SUBSCRIPTION_RESUMED: MESubscriptionResumed,
    MemberpressEvents.SUBSCRIPTION_STOPPED: MESubscriptionStopped,
    MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_ONE_TIME: MESubscriptionUpgradedToOneTime,
    MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_RECURRING: MESubscriptionUpgradedToRecurring,
    MemberpressEvents.SUBSCRIPTION_UPGRADED: MESubscriptionUpgraded,
    MemberpressEvents.TRANSACTION_COMPLETED: METransactionCompleted,
    MemberpressEvents.TRANSACTION_EXPIRED: METransactionExpired,
    MemberpressEvents.TRANSACTION_FAILED: METransactionFailed,
    MemberpressEvents.TRANSACTION_REFUNDED: METransactionRefunded,
    MemberpressEvents.UNIDENTIFIED_EVENT: MEUnidentifiedEvent,
}


def get_event(data: dict) -> MemberpressEventChild:
    """
    introspect a data dict received by a memberpress webhook event, determine the event type
    and return an instance of the corresponding class.
    """
    event = data.get("event", None) or MemberpressEvents.UNIDENTIFIED_EVENT
    cls = MEMBERPRESS_EVENT_CLASSES[event]
    return cls.factory(data=data)



from django.contrib import admin
from memberpress_client.models import MemberpressEventLog


class MemberpressEventLogAdmin(admin.ModelAdmin):
    """
    Memberpress Webhook event log
    """

    def has_change_permission(self, request, obj=None):
        return False

    search_fields = ()
    list_display = (
        "created",
        "event",
        "event_type",
        "is_valid",
        "is_processed",
        "username",
        "json",
    )


admin.site.register(MemberpressEventLog, MemberpressEventLogAdmin)



# Generated by Django 3.2.16 on 2022-12-19 03:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('memberpress_client', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='memberpresseventlog',
            options={'verbose_name_plural': 'memberpress event log'},
        ),
        migrations.AlterField(
            model_name='memberpresseventlog',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - App Configuration
"""
# python stuff
import logging

# django stuff
from django.apps import AppConfig


try:
    # hooks for openedx plugins
    # these imports only work when this plugin is installed
    # in an Open edX build.
    from edx_django_utils.plugins import PluginSettings, PluginURLs
    from openedx.core.djangoapps.plugins.constants import ProjectType, SettingsType
except ImportError:
    # for both development as well as deployment into a
    # non open edx Django platform.
    class PluginSettings:
        CONFIG = "config"
        RELATIVE_PATH = "relative/path/"

    class PluginURLs:
        CONFIG = "config"
        NAMESPACE = "namespace"
        REGEX = ("regex",)
        RELATIVE_PATH = "urls"

    class ProjectType:
        LMS = ""
        CMS = ""

    class SettingsType:
        PRODUCTION = ""
        COMMON = ""


log = logging.getLogger(__name__)


class MemberPressPluginConfig(AppConfig):
    name = "memberpress_client"
    label = "memberpress_client"

    # This is the text that appears in the Django admin console in all caps
    # as the title box encapsulating all Django app models that are registered
    # in admin.py.
    verbose_name = "memberpress REST API Client plugin"

    # See: https://edx.readthedocs.io/projects/edx-django-utils/en/latest/edx_django_utils.plugins.html
    plugin_app = {
        PluginURLs.CONFIG: {
            ProjectType.LMS: {
                PluginURLs.NAMESPACE: name,
                PluginURLs.REGEX: "^mp/",
                PluginURLs.RELATIVE_PATH: "urls",
            }
        },
        PluginSettings.CONFIG: {
            ProjectType.LMS: {
                SettingsType.PRODUCTION: {PluginSettings.RELATIVE_PATH: "settings.production"},
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.common"},
            }
        },
    }

    def ready(self):
        log.info("{label} is ready.".format(label=self.label))



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - rest api client implementation.
This is the base class for memberpress Classes.
"""
# Python stuff
import logging
import inspect
import json
import urllib3
from urllib.parse import urljoin
import requests

# Django stuff
from django.conf import settings
from django.core.cache import cache

# our stuff
from memberpress_client.memberpress import Memberpress
from memberpress_client.utils import log_pretrip, log_postrip
from memberpress_client.decorators import request_manager

# disable the following warnings:
# -------------------------------
# /usr/local/lib/python3.9/site-packages/urllib3/connectionpool.py:1043:
# InsecureRequestWarning: Unverified HTTPS request is being made to host 'staging.global-communications-academy.com'.
# Adding certificate verification is strongly advised. See: https://urllib3.readtheocs.io/en/1.26.x/advanced-usage.html#ssl-warnings
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logger = logging.getLogger(__name__)


class MemberpressAPIClient(Memberpress):
    def init(self):
        super().init()

    def get_url(self, path) -> str:
        return urljoin(settings.MEMBERPRESS_API_BASE_URL, path)

    @property
    def headers(self) -> dict:
        return {
            f"{settings.MEMBERPRESS_API_KEY_NAME}": f"{settings.MEMBERPRESS_API_KEY}",
        }

    @request_manager
    def post(self, path, data=None, host=None, operation="") -> json:
        url = self.get_url(path, host=host)
        log_pretrip(caller=inspect.currentframe().f_code.co_name, url=url, data=data, operation=operation)
        response = requests.post(url, data=data, headers=self.headers)
        log_postrip(caller=inspect.currentframe().f_code.co_name, path=url, response=response, operation=operation)
        response.raise_for_status()
        return response.json()

    @request_manager
    def patch(self, path, data=None, host=None, headers=None, json=True, operation=""):
        url = self.get_url(path, host=host)
        if not headers:
            headers = self.headers

        log_pretrip(caller=inspect.currentframe().f_code.co_name, url=url, data=data, operation=operation)
        response = requests.patch(url, json=data, headers=headers)
        log_postrip(caller=inspect.currentframe().f_code.co_name, path=url, response=response, operation=operation)
        response.raise_for_status()
        if json:
            return response.json()
        return response

    @request_manager
    def get(self, path, params=None, operation="", enable_caching=True) -> json:
        url = self.get_url(path)
        response = None
        if enable_caching:
            cache_key_params = json.dumps(params, sort_keys=True, separators=(',', ':'))
            cache_key = f"MemberpressAPIClient.get:{url}:{cache_key_params}"
            response = cache.get(cache_key)

        if not response and not self.locked:
            # set a lock to prevent multiple calls
            self.lock()

            # purge whatever might have previously existing in the cache, in case our
            # response object raises an exception here ...
            if enable_caching:
                cache.delete(cache_key)

            log_pretrip(caller=inspect.currentframe().f_code.co_name, url=url, data={}, operation=operation)
            response = requests.get(url, params=params, headers=self.headers, verify=False)
            log_postrip(caller=inspect.currentframe().f_code.co_name, path=url, response=response, operation=operation)

            # @request_manager will create verbose log entries for any responses outside of 200-299.
            response.raise_for_status()

            if type(response) not in [dict, list]:
                try:
                    response = response.json()
                except Exception:
                    response = json.dumps(response)

            # caching results iff response is a valid json object.
            if enable_caching:
                cache.set(cache_key, response, settings.MEMBERPRESS_CACHE_EXPIRATION)
            self.unlock()
        return response

    def is_valid_dict(self, response, qc_keys) -> bool:
        if not type(response) == dict:
            logger.warning(
                "is_valid_dict() was expecting a dict but received an object of type: {type}".format(
                    type=type(response)
                )
            )
            return False
        return all(key in response for key in qc_keys)



"""
Common Pluggable Django App settings

Handling of environment variables, see: https://django-environ.readthedocs.io/en/latest/
to convert .env to yml see: https://django-environ.readthedocs.io/en/latest/tips.html#docker-style-file-based-variables
"""
from path import Path as path
import environ
import os

env = environ.Env(
    # set casting, default value
    MEMBERPRESS_API_KEY=(str, "set-me-please"),
    MEMBERPRESS_API_BASE_URL=(str, "https://yourdomain.com"),
    MEMBERPRESS_API_KEY_NAME=(str, "MEMBERPRESS-API-KEY"),
    MEMBERPRESS_CACHE_EXPIRATION=(int, 60 * 60 * 24),
    MEMBERPRESS_SENSITIVE_KEYS=(
        list,
        [
            "password",
            "token",
            "client_id",
            "client_secret",
            "Authorization",
            "secret",
        ],
    ),
)

# path to this file.
HERE = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
APP_ROOT = path(__file__).abspath().dirname().dirname()  # /blah/blah/blah/.../memberpress_client
REPO_ROOT = APP_ROOT.dirname()  # /blah/blah/blah/.../django-memberpress-client
TEMPLATES_DIR = APP_ROOT / "templates"

environ.Env.read_env(os.path.join(REPO_ROOT, ".env"))


def plugin_settings(settings):
    """
    Injects local settings into django settings
    """

    settings.MEMBERPRESS_API_KEY = env("MEMBERPRESS_API_KEY")
    settings.MEMBERPRESS_API_BASE_URL = env("MEMBERPRESS_API_BASE_URL")
    settings.MEMBERPRESS_API_KEY_NAME = env("MEMBERPRESS_API_KEY_NAME")  # noqa: F841
    settings.MEMBERPRESS_CACHE_EXPIRATION = env("MEMBERPRESS_CACHE_EXPIRATION")  # noqa: F841
    settings.MEMBERPRESS_SENSITIVE_KEYS = env("MEMBERPRESS_SENSITIVE_KEYS")  # noqa: F841

    settings.MAKO_TEMPLATE_DIRS_BASE.extend([TEMPLATES_DIR])



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - plugin constants
"""

# django stuff
from django.conf import settings
from django.utils.functional import classproperty


class MemberpressTransactionTypes:
    SUBSCRIPTION_CONFIRMATION = "subscription_confirmation"
    PAYMENT = "payment"


class TypeBase:
    @classmethod
    def all(self):
        """
        generate a list of all class variable values
        """
        return [
            getattr(self, value)
            for value in [attr for attr in dir(self) if not callable(getattr(self, attr)) and not attr.startswith("__")]
        ]


class MemberpressEventTypes(TypeBase):
    TRANSACTION = "transaction"
    SUBSCRIPTION = "subscription"
    MEMBER = "member"
    MEMBERSHIP = "membership"


class MemberpressEvents(TypeBase):
    AFTER_CC_EXPIRES_REMINDER = "after-cc-expires-reminder"
    AFTER_MEMBER_SIGNUP_REMINDER = "after-member-signup-reminder"
    AFTER_SIGNUP_ABANDONED_REMINDER = "after-signup-abandoned-reminder"
    AFTER_SUB_EXPIRES_REMINDER = "after-sub-expires-reminder"
    BEFORE_CC_EXPIRES_REMINDER = "before-cc-expires-reminder"
    BEFORE_SUB_EXPIRES_REMINDER = "before-sub-expires-reminder"
    BEFORE_SUB_RENEWS_REMINDER = "before-sub-renews-reminder"
    BEFORE_SUB_TRIAL_ENDS = "before-sub-trial-ends"
    LOGIN = "login"
    MEMBER_ACCOUNT_UPDATED = "member-account-updated"
    MEMBER_ADDED = "member-added"
    MEMBER_DELETED = "member-deleted"
    MEMBER_SIGNUP_COMPLETED = "member-signup-completed"
    MPCA_COURSE_COMPLETED = "mpca-course-completed"
    MPCA_COURSE_STARTED = "mpca-course-started"
    MPCA_LESSON_COMPLETED = "mpca-lesson-completed"
    MPCA_LESSON_STARTED = "mpca-lesson-started"
    MPCA_QUIZ_ATTEMPT_COMPLETED = "mpca-quiz-attempt-completed"
    NON_RECURRING_TRANSACTION_COMPLETED = "non-recurring-transaction-completed"
    NON_RECURRING_TRANSACTION_EXPIRED = "non-recurring-transaction-expired"
    OFFLINE_PAYMENT_COMPLETE = "offline-payment-complete"
    OFFLINE_PAYMENT_PENDING = "offline-payment-pending"
    OFFLINE_PAYMENT_REFUNDED = "offline-payment-refunded"
    RECURRING_TRANSACTION_COMPLETED = "recurring-transaction-completed"
    RECURRING_TRANSACTION_EXPIRED = "recurring-transaction-expired"
    RECURRING_TRANSACTION_FAILED = "recurring-transaction-failed"
    RENEWAL_TRANSACTION_COMPLETED = "renewal-transaction-completed"
    SUB_ACCOUNT_ADDED = "sub-account-added"
    SUB_ACCOUNT_REMOVED = "sub-account-removed"
    SUBSCRIPTION_CREATED = "subscription-created"
    SUBSCRIPTION_DOWNGRADED_TO_ONE_TIME = "subscription-downgraded-to-one-time"
    SUBSCRIPTION_DOWNGRADED_TO_RECURRING = "subscription-downgraded-to-recurring"
    SUBSCRIPTION_DOWNGRADED = "subscription-downgraded"
    SUBSCRIPTION_EXPIRED = "subscription-expired"
    SUBSCRIPTION_PAUSED = "subscription-paused"
    SUBSCRIPTION_RESUMED = "subscription-resumed"
    SUBSCRIPTION_STOPPED = "subscription-stopped"
    SUBSCRIPTION_UPGRADED_TO_ONE_TIME = "subscription-upgraded-to-one-time"
    SUBSCRIPTION_UPGRADED_TO_RECURRING = "subscription-upgraded-to-recurring"
    SUBSCRIPTION_UPGRADED = "subscription-upgraded"
    TRANSACTION_COMPLETED = "transaction-completed"
    TRANSACTION_EXPIRED = "transaction-expired"
    TRANSACTION_FAILED = "transaction-failed"
    TRANSACTION_REFUNDED = "transaction-refunded"
    UNIDENTIFIED_EVENT = "unidentified-event"


MEMBERPRESS_EVENT_TYPES = (
    (MemberpressEventTypes.MEMBER, MemberpressEventTypes.MEMBER),
    (MemberpressEventTypes.MEMBERSHIP, MemberpressEventTypes.MEMBERSHIP),
    (MemberpressEventTypes.SUBSCRIPTION, MemberpressEventTypes.SUBSCRIPTION),
    (MemberpressEventTypes.TRANSACTION, MemberpressEventTypes.TRANSACTION),
)

MEMBERPRESS_EVENTS = (
    (MemberpressEvents.AFTER_CC_EXPIRES_REMINDER, MemberpressEvents.AFTER_CC_EXPIRES_REMINDER),
    (MemberpressEvents.AFTER_MEMBER_SIGNUP_REMINDER, MemberpressEvents.AFTER_MEMBER_SIGNUP_REMINDER),
    (MemberpressEvents.AFTER_SIGNUP_ABANDONED_REMINDER, MemberpressEvents.AFTER_SIGNUP_ABANDONED_REMINDER),
    (MemberpressEvents.AFTER_SUB_EXPIRES_REMINDER, MemberpressEvents.AFTER_SUB_EXPIRES_REMINDER),
    (MemberpressEvents.BEFORE_CC_EXPIRES_REMINDER, MemberpressEvents.BEFORE_CC_EXPIRES_REMINDER),
    (MemberpressEvents.BEFORE_SUB_EXPIRES_REMINDER, MemberpressEvents.BEFORE_SUB_EXPIRES_REMINDER),
    (MemberpressEvents.BEFORE_SUB_RENEWS_REMINDER, MemberpressEvents.BEFORE_SUB_RENEWS_REMINDER),
    (MemberpressEvents.BEFORE_SUB_TRIAL_ENDS, MemberpressEvents.BEFORE_SUB_TRIAL_ENDS),
    (MemberpressEvents.LOGIN, MemberpressEvents.LOGIN),
    (MemberpressEvents.MEMBER_ACCOUNT_UPDATED, MemberpressEvents.MEMBER_ACCOUNT_UPDATED),
    (MemberpressEvents.MEMBER_ADDED, MemberpressEvents.MEMBER_ADDED),
    (MemberpressEvents.MEMBER_DELETED, MemberpressEvents.MEMBER_DELETED),
    (MemberpressEvents.MEMBER_SIGNUP_COMPLETED, MemberpressEvents.MEMBER_SIGNUP_COMPLETED),
    (MemberpressEvents.MPCA_COURSE_COMPLETED, MemberpressEvents.MPCA_COURSE_COMPLETED),
    (MemberpressEvents.MPCA_COURSE_STARTED, MemberpressEvents.MPCA_COURSE_STARTED),
    (MemberpressEvents.MPCA_LESSON_COMPLETED, MemberpressEvents.MPCA_LESSON_COMPLETED),
    (MemberpressEvents.MPCA_LESSON_STARTED, MemberpressEvents.MPCA_LESSON_STARTED),
    (MemberpressEvents.MPCA_QUIZ_ATTEMPT_COMPLETED, MemberpressEvents.MPCA_QUIZ_ATTEMPT_COMPLETED),
    (MemberpressEvents.NON_RECURRING_TRANSACTION_COMPLETED, MemberpressEvents.NON_RECURRING_TRANSACTION_COMPLETED),
    (MemberpressEvents.NON_RECURRING_TRANSACTION_EXPIRED, MemberpressEvents.NON_RECURRING_TRANSACTION_EXPIRED),
    (MemberpressEvents.OFFLINE_PAYMENT_COMPLETE, MemberpressEvents.OFFLINE_PAYMENT_COMPLETE),
    (MemberpressEvents.OFFLINE_PAYMENT_PENDING, MemberpressEvents.OFFLINE_PAYMENT_PENDING),
    (MemberpressEvents.OFFLINE_PAYMENT_REFUNDED, MemberpressEvents.OFFLINE_PAYMENT_REFUNDED),
    (MemberpressEvents.RECURRING_TRANSACTION_COMPLETED, MemberpressEvents.RECURRING_TRANSACTION_COMPLETED),
    (MemberpressEvents.RECURRING_TRANSACTION_EXPIRED, MemberpressEvents.RECURRING_TRANSACTION_EXPIRED),
    (MemberpressEvents.RECURRING_TRANSACTION_FAILED, MemberpressEvents.RECURRING_TRANSACTION_FAILED),
    (MemberpressEvents.RENEWAL_TRANSACTION_COMPLETED, MemberpressEvents.RENEWAL_TRANSACTION_COMPLETED),
    (MemberpressEvents.SUB_ACCOUNT_ADDED, MemberpressEvents.SUB_ACCOUNT_ADDED),
    (MemberpressEvents.SUB_ACCOUNT_REMOVED, MemberpressEvents.SUB_ACCOUNT_REMOVED),
    (MemberpressEvents.SUBSCRIPTION_CREATED, MemberpressEvents.SUBSCRIPTION_CREATED),
    (MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_ONE_TIME, MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_ONE_TIME),
    (MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_RECURRING, MemberpressEvents.SUBSCRIPTION_DOWNGRADED_TO_RECURRING),
    (MemberpressEvents.SUBSCRIPTION_DOWNGRADED, MemberpressEvents.SUBSCRIPTION_DOWNGRADED),
    (MemberpressEvents.SUBSCRIPTION_EXPIRED, MemberpressEvents.SUBSCRIPTION_EXPIRED),
    (MemberpressEvents.SUBSCRIPTION_PAUSED, MemberpressEvents.SUBSCRIPTION_PAUSED),
    (MemberpressEvents.SUBSCRIPTION_RESUMED, MemberpressEvents.SUBSCRIPTION_RESUMED),
    (MemberpressEvents.SUBSCRIPTION_STOPPED, MemberpressEvents.SUBSCRIPTION_STOPPED),
    (MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_ONE_TIME, MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_ONE_TIME),
    (MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_RECURRING, MemberpressEvents.SUBSCRIPTION_UPGRADED_TO_RECURRING),
    (MemberpressEvents.SUBSCRIPTION_UPGRADED, MemberpressEvents.SUBSCRIPTION_UPGRADED),
    (MemberpressEvents.TRANSACTION_COMPLETED, MemberpressEvents.TRANSACTION_COMPLETED),
    (MemberpressEvents.TRANSACTION_EXPIRED, MemberpressEvents.TRANSACTION_EXPIRED),
    (MemberpressEvents.TRANSACTION_FAILED, MemberpressEvents.TRANSACTION_FAILED),
    (MemberpressEvents.TRANSACTION_REFUNDED, MemberpressEvents.TRANSACTION_REFUNDED),
    (MemberpressEvents.UNIDENTIFIED_EVENT, MemberpressEvents.UNIDENTIFIED_EVENT),
)


MEMBERPRESS_OPERATION_PREFIX = "memberpress_api_operation_"
OPERATION_GET_MEMBER = MEMBERPRESS_OPERATION_PREFIX + "get_member"


class MemberPressAPI_Operations:
    __slots__ = ()
    GET_MEMBER = OPERATION_GET_MEMBER


class MemberPressAPI_Endpoints:
    """
    written by: mcdaniel
    date:       oct-2022
    Codify the data models of the api endpoints and data dicts
    referenced by MemberPress REST API
    """

    # -------------------------------------------------------------------------
    # api end points originating from https://stepwisemath.ai/wp-json/mp/v1/
    # -------------------------------------------------------------------------

    @classproperty
    def MEMBERPRESS_API_BASE(cls):
        return f"{settings.MEMBERPRESS_API_BASE_URL}wp-json/mp/v1/"

    @classproperty
    def MEMBERPRESS_API_ME_PATH(cls):
        return f"{cls.MEMBERPRESS_API_BASE}me/"

    # -------------------------------------------------------------------------
    # curl "https://set-me-please.com/wp-json/mp/v1/members?search=mcdaniel" -H "MEMBERPRESS-API-KEY: set-me-please"
    # -------------------------------------------------------------------------
    def MEMBERPRESS_API_MEMBER_PATH(username):
        return MemberPressAPI_Endpoints.MEMBERPRESS_API_BASE + "members?search=" + username


COMPLETE_MEMBER_DICT = [
    "id",
    "email",
    "username",
    "nicename",
    "url",
    "message",
    "registered_at",
    "first_name",
    "last_name",
    "display_name",
    "active_memberships",
    "active_txn_count",
    "expired_txn_count",
    "trial_txn_count",
    "sub_count",
    "login_count",
    "first_txn",
    "latest_txn",
    "address",
    "profile",
    "recent_transactions",
    "recent_subscriptions",
]
MINIMUM_MEMBER_DICT = ["username", "recent_transactions", "recent_subscriptions", "active_memberships"]

PARTIAL_MEMBER_DICT = [
    "id",
    "email",
    "username",
    "nicename",
    "url",
    "message",
    "registered_at",
    "first_name",
    "last_name",
    "display_name",
    "active_memberships",
    "active_txn_count",
    "expired_txn_count",
    "trial_txn_count",
    "sub_count",
    "login_count",
    "address",
    "profile",
    "recent_transactions",
    "recent_subscriptions",
]


COMPLETE_SUBSCRIPTION_DICT = [
    "coupon",
    "membership",
    "member",
    "id",
    "subscr_id",
    "gateway",
    "price",
    "period",
    "period_type",
    "limit_cycles",
    "limit_cycles_num",
    "limit_cycles_action",
    "limit_cycles_expires_after",
    "limit_cycles_expires_type",
    "prorated_trial",
    "trial",
    "trial_days",
    "trial_amount",
    "trial_tax_amount",
    "trial_total",
    "status",
    "created_at",
    "total",
    "tax_rate",
    "tax_amount",
    "tax_desc",
    "tax_class",
    "cc_last4",
    "cc_exp_month",
    "cc_exp_year",
    "token",
    "tax_compound",
    "tax_shipping",
    "response",
]

COMPLETE_TRANSACTION_DICT = [
    "membership",
    "member",
    "coupon",
    "subscription",
    "id",
    "amount",
    "total",
    "tax_amount",
    "tax_rate",
    "tax_desc",
    "tax_class",
    "trans_num",
    "status",
    "txn_type",
    "gateway",
    "prorated",
    "created_at",
    "expires_at",
    "corporate_account_id",
    "parent_transaction_id",
    "tax_compound",
    "tax_shipping",
    "response",
]

COMPLETE_EVENT_DICT = [
    "coupon",
    "id",
    "subscr_id",
    "gateway",
    "price",
    "period",
    "period_type",
    "limit_cycles",
    "limit_cycles_num",
    "limit_cycles_action",
    "limit_cycles_expires_after",
    "limit_cycles_expires_type",
    "prorated_trial",
    "trial",
    "trial_days",
    "trial_amount",
    "trial_tax_amount",
    "trial_total",
    "status",
    "created_at",
    "total",
    "tax_rate",
    "tax_amount",
    "tax_desc",
    "tax_class",
    "cc_last4",
    "cc_exp_month",
    "cc_exp_year",
    "token",
    "tax_compound",
    "tax_shipping",
    "response",
]

COMPLETE_MEMBERSHIP_DICT = [
    "id",
    "title",
    "content",
    "excerpt",
    "date",
    "status",
    "author",
    "date_gmt",
    "modified",
    "modified_gmt",
    "group",
    "price",
    "period",
    "period_type",
    "signup_button_text",
    "limit_cycles",
    "limit_cycles_num",
    "limit_cycles_action",
    "limit_cycles_expires_after",
    "limit_cycles_expires_type",
    "trial",
    "trial_days",
    "trial_amount",
    "trial_once",
    "group_order",
    "is_highlighted",
    "plan_code",
    "pricing_title",
    "pricing_show_price",
    "pricing_display",
    "custom_price",
    "pricing_heading_txt",
    "pricing_footer_txt",
    "pricing_button_txt",
    "pricing_button_position",
    "pricing_benefits",
    "register_price_action",
    "register_price",
    "thank_you_page_enabled",
    "thank_you_page_type",
    "thank_you_message",
    "thank_you_page_id",
    "custom_login_urls_enabled",
    "custom_login_urls_default",
    "custom_login_urls",
    "expire_type",
    "expire_after",
    "expire_unit",
    "expire_fixed",
    "tax_exempt",
    "tax_class",
    "allow_renewal",
    "access_url",
    "disable_address_fields",
    "simultaneous_subscriptions",
    "use_custom_template",
    "custom_template",
    "customize_payment_methods",
    "custom_payment_methods",
    "customize_profile_fields",
    "custom_profile_fields",
    "cannot_purchase_message",
]

COMPLETE_FINANCIAL_SUBSCRIPTION_EVENT = [
    "coupon",
    "id",
    "amount",
    "total",
    "tax_amount",
    "tax_rate",
    "tax_desc",
    "tax_class",
    "trans_num",
    "status",
    "txn_type",
    "gateway",
    "prorated",
    "created_at",
    "expires_at",
    "corporate_account_id",
    "parent_transaction_id",
    "tax_compound",
    "tax_shipping",
    "response",
    "rebill",
    "subscription_payment_index",
]

COMPLETE_NONFINANCIAL_SUBSCRIPTION_EVENT = [
    "coupon",
    "id",
    "subscr_id",
    "gateway",
    "price",
    "period",
    "period_type",
    "limit_cycles",
    "limit_cycles_num",
    "limit_cycles_action",
    "limit_cycles_expires_after",
    "limit_cycles_expires_type",
    "prorated_trial",
    "trial",
    "trial_days",
    "trial_amount",
    "trial_tax_amount",
    "trial_total",
    "status",
    "created_at",
    "total",
    "tax_rate",
    "tax_amount",
    "tax_desc",
    "tax_class",
    "cc_last4",
    "cc_exp_month",
    "cc_exp_year",
    "token",
    "tax_compound",
    "tax_shipping",
    "response",
]



# pylint: disable=open-builtin
import io
import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

HERE = os.path.abspath(os.path.dirname(__file__))


def load_readme():
    with io.open(os.path.join(HERE, "README.md"), "rt", encoding="utf8") as f:
        return f.read()


README = load_readme()


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns:
        list: Requirements file relative path strings
    """
    requirements = set()
    for path in requirements_paths:
        requirements.update(
            line.split("#")[0].strip() for line in open(path).readlines() if is_requirement(line.strip())
        )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement.
    Returns:
        bool: True if the line is not blank, a comment, a URL, or an included file
    """
    return not (
        line == ""
        or line.startswith("-c")
        or line.startswith("-r")
        or line.startswith("#")
        or line.startswith("-e")
        or line.startswith("git+")
    )


print("Found packages: {packages}".format(packages=find_packages()))

print("requirements found: {requirements}".format(requirements=load_requirements("requirements/common.in")))

setup(
    name="django-memberpress-client",
    url="https://github.com/lpm0073/django-memberpress-client",
    project_urls={
        "Code": "https://github.com/lpm0073/django-memberpress-client",
        "Issue tracker": "https://github.com/lpm0073/django-memberpress-client/issues",
        "Community": "https://docs.memberpress.com/category/215-developer-resources",
    },
    author="Lawrence McDaniel",
    author_email="lpm0073@gmail.com",
    description="A Django plugin to add Memberpress REST API and Webhook integrations.",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    package_data={"": ["*.html"]},  # include any Mako templates found in this repo.
    zip_safe=False,
    keywords="Python, Django, Wordpress, MemberPress, REST API",
    python_requires=">=3.8",
    install_requires=load_requirements("requirements/common.txt"),
    entry_points={
        # mcdaniel aug-2021
        #
        # IMPORTANT: ensure that this entry_points coincides with that of edx-platform
        #            and also that you are not introducing any name collisions.
        # https://github.com/openedx/edx-platform/blob/master/setup.py#L88
        "lms.djangoapp": [
            "memberpress_client = memberpress_client.apps:MemberPressPluginConfig",
        ],
        "cms.djangoapp": [],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - decorators.
request_manager() - common logging and error handling for all REST api verbs
app_logger() - better logging for lms.log and cms.log
"""
# python stuff
import json
import functools
import logging
from requests.exceptions import HTTPError

# our stuff
from .utils import MPJSONEncoder, masked_dict

# module initializations
logger = logging.getLogger(__name__)


def request_manager(method):
    """
    Decorate a method to
    - retry on 401 http exceptions. Will attempt to refresh the token in this case, and try again.
    - catch and kill 429 exceptions.
    - catch and kill 415 exceptions.
    """

    def wrapper(*args, **kwargs):
        cls = args[0]  # noqa: F841
        operation = ""

        try:
            return method(*args, **kwargs)
        except HTTPError as e:
            # look for an operation description if it exists
            if "operation" in kwargs.keys():
                operation = " " + kwargs["operation"]

            # treat any exception as unhandled. however, we should add meta
            # data about the request to the stack trace. dump the request
            # headers and body using MPJSONEncoder so that
            # sensitive data is masked.
            request_body = json.dumps(e.response.content, cls=MPJSONEncoder, indent=4) if e.response is not None else ""
            request_headers = (
                json.dumps(masked_dict(dict(e.response.request.headers)), cls=MPJSONEncoder, indent=4)
                if e.response is not None and e.response.request is not None
                else ""
            )
            raise Exception(
                "memberpress_client.decorators.request_manager(){operation} an unhandled exception '{error_message}', was returned by {method}(): {verb} {url}, headers={headers}, body={body}".format(
                    operation=operation,
                    method=method.__name__,
                    verb=e.response.request.method,
                    url=e.response.request.url,
                    headers=request_headers,
                    body=request_body,
                    error_message=str(e),
                )
            ) from e

    return wrapper


def app_logger(func):
    """
    Decorate a function to add an entry to the app log with the function name,
    its positional arguments, and keyword pairs presented as a formatted dict.

    sample output:
        2022-10-07 19:45:26,869 INFO app_logger: registration.views.EmailVerificationView().get() ["<WSGIRequest: GET '/verify-email/MjY1/bcxw75-69581da3ea4f0cefad2f0f2205354117/'>"] keyword args: {
            "uid": "MjY1",
            "token": "*** -- REDACTED -- ***"
        }
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        name_of_def = func.__name__
        logged_args = args
        kwargs_dict_repr = ""

        # try initializing variables assuming that we were called by a class method
        try:
            cls = args[0].__class__
            name_of_class = cls.__name__ + "()."
            name_of_module = cls.__module__
            # slice off the 'self' positional argument
            logged_args = args[1:]
        except Exception:
            # We weren't called by a class method. Fall back to initializing variables for
            # a standard module function.
            name_of_class = ""
            name_of_module = func.__module__

        positional_args = [repr(a) for a in logged_args]

        if len(kwargs.keys()) > 0:
            kwargs_dict_repr = "keyword args: "
            kwargs_dict_repr += json.dumps(masked_dict(kwargs), cls=MPJSONEncoder, indent=4)

        logger.info(
            "app_logger: {name_of_module}.{name_of_class}{name_of_def}() {args} {kwargs}".format(
                name_of_module=name_of_module,
                name_of_class=name_of_class,
                name_of_def=name_of_def,
                args=positional_args if len(positional_args) > 0 else "",
                kwargs=kwargs_dict_repr,
            )
        )
        return func(*args, **kwargs)

    return wrapper



"""
Lawrence McDaniel - https://lawrencemcdaniel.com
Oct-2022

memberpress REST API Client plugin for Django - Models.
"""
from email.policy import default
from django.utils.translation import ugettext as _
from django.db import models
from model_utils.models import TimeStampedModel

# To do: remove this when Open edX moves to Django 3.x
from jsonfield import JSONField

from memberpress_client.constants import MEMBERPRESS_EVENTS, MEMBERPRESS_EVENT_TYPES


class MemberpressEventLog(TimeStampedModel):
    """ """

    class Meta:
        verbose_name_plural = "memberpress event log"

    sender = models.URLField(
        blank=True,
        help_text=_("The site referrer. Example: https://wordpress-site.com/mb/webhooks/"),
    )

    username = models.CharField(
        blank=False,
        max_length=50,
        help_text=_("The username provided by memberpress."),
    )

    event = models.CharField(
        blank=False,
        choices=MEMBERPRESS_EVENTS,
        max_length=50,
        help_text=_("The memberpress event. Examples: recurring-transaction-completed"),
    )

    event_type = models.CharField(
        blank=False,
        choices=MEMBERPRESS_EVENT_TYPES,
        max_length=50,
        help_text=_("The type of memberpress event: transaction, subscription, member, membership"),
    )

    is_valid = models.BooleanField(
        blank=False,
        help_text=_("True if the json received was validated by memberpress_client. False otherwise."),
    )

    is_processed = models.BooleanField(
        blank=True,
        default=False,
        help_text=_("True if this event has been analyzed and acted upon."),
    )

    json = JSONField(
        blank=True,
        default={},
        help_text=_("A json dict sent by the webhook event in the request body."),
    )

    def __str__(self):
        return str(self.created) + "-" + self.event



# python stuff
from asyncio.log import logger
import os
import io
import unittest
import json
from datetime import datetime

# our testing code starts here
# -----------------------------------------------------------------------------
from memberpress_client.constants import MemberpressEvents, MemberpressTransactionTypes
from memberpress_client.events import get_event, MEMBERPRESS_EVENT_CLASSES
from memberpress_client.models import MemberpressEventLog

# setup test data
HERE = os.path.abspath(os.path.dirname(__file__))
EVENTS_FOLDER = os.path.join(HERE, "data", "events")
EXT = (".json",)


def load_json(test_file):
    if test_file == "unidentified-event":
        return

    test_file = test_file + ".json"
    with io.open(os.path.join(EVENTS_FOLDER, test_file), "rt", encoding="utf8") as f:
        return json.loads(f.read(), strict=False)


class TestMember(unittest.TestCase):
    def test_valid_dicts(self):
        def validate(event_str: str):
            data_dict = load_json(event_str)
            if not data_dict:
                logger.warning("no test file for {event_str}".format(event_str=event_str))
                return

            event = get_event(data_dict)
            self.assertEqual(event.is_valid, True)
            self.assertEqual(event.event, event_str)
            self.assertEqual(type(event), MEMBERPRESS_EVENT_CLASSES[event_str])

        for file in os.listdir(EVENTS_FOLDER):
            if file.endswith(EXT):
                validate(event_str=file[:-5])

        # test MemberpressEvents.all()
        for event_str in MemberpressEvents.all():
            validate(event_str=event_str)

    def test_valid_login_event(self):
        event_str = MemberpressEvents.LOGIN
        data_dict = load_json(event_str)
        event = get_event(data_dict)

        # tests
        self.assertEqual(event.is_valid, True)
        self.assertEqual(event.event, event_str)
        self.assertEqual(type(event), MEMBERPRESS_EVENT_CLASSES[event_str])

        self.assertEqual(event.id, 6)
        self.assertEqual(event.email, "lpm0073@gmail.com")
        self.assertEqual(event.username, "mcdaniel")
        self.assertEqual(event.nicename, "mcdaniel")
        self.assertEqual(event.url, "https://lawrencemcdaniel.com")
        self.assertEqual(event.message, "")
        self.assertEqual(event.registered_at, datetime.strptime("2022-10-04 00:46:37", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(event.first_name, "Lawrence")
        self.assertEqual(event.last_name, "McDaniel")
        self.assertEqual(event.active_memberships, [])
        self.assertEqual(event.active_txn_count, 0)
        self.assertEqual(event.expired_txn_count, 0)
        self.assertEqual(event.trial_txn_count, 0)
        self.assertEqual(event.sub_count, None)
        self.assertEqual(event.login_count, 25)
        self.assertEqual(type(event.address), dict)
        self.assertEqual(len(event.address.keys()), 6)
        self.assertEqual(type(event.profile), dict)
        self.assertEqual(event.profile, {})
        self.assertEqual(event.recent_transactions, [])
        self.assertEqual(event.recent_subscriptions, [])

    def test_valid_payment_event(self):
        event_str = MemberpressEvents.OFFLINE_PAYMENT_COMPLETE
        data_dict = load_json(event_str)
        event = get_event(data_dict)

        # tests
        self.assertEqual(event.is_valid, True)
        self.assertEqual(event.event, event_str)
        self.assertEqual(type(event), MEMBERPRESS_EVENT_CLASSES[event_str])
        self.assertEqual(event.id, 3)
        self.assertEqual(event.amount, 9.00)
        self.assertEqual(event.total, 9.00)
        self.assertEqual(event.tax_amount, 0.00)
        self.assertEqual(event.tax_rate, 0.000)
        self.assertEqual(event.tax_desc, "")
        self.assertEqual(event.tax_class, "standard")
        self.assertEqual(event.trans_num, "seti_1LsvYgJ1UGflvSOWgLojGVhy")
        self.assertEqual(event.status, "pending")
        self.assertEqual(event.txn_type, MemberpressTransactionTypes.PAYMENT)
        self.assertEqual(event.gateway, "rj1l52-6bw")
        self.assertEqual(event.prorated, 0)
        self.assertEqual(event.created_at, datetime.strptime("2022-10-14 21:36:42", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(event.expires_at, datetime.strptime("2022-11-14 23:59:59", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(event.corporate_account_id, 0)
        self.assertEqual(event.parent_transaction_id, 0)
        self.assertEqual(event.tax_compound, 0)
        self.assertEqual(event.tax_shipping, True)
        self.assertEqual(event.response, None)
        self.assertEqual(event.rebill, False)
        self.assertEqual(event.subscription_payment_index, False)

    def test_valid_subscription_event(self):
        event_str = MemberpressEvents.SUBSCRIPTION_CREATED
        data_dict = load_json(event_str)
        event = get_event(data_dict)

        # tests
        self.assertEqual(event.is_valid, True)
        self.assertEqual(event.event, event_str)
        self.assertEqual(type(event), MEMBERPRESS_EVENT_CLASSES[event_str])
        self.assertEqual(event.id, 3)
        self.assertEqual(event.subscr_id, "mp-sub-6349d66a5744f")
        self.assertEqual(event.gateway, "rj1l52-6bw")
        self.assertEqual(event.price, 9.00)
        self.assertEqual(event.period, 1)
        self.assertEqual(event.period_type, "months")
        self.assertEqual(event.limit_cycles, False)
        self.assertEqual(event.limit_cycles_num, 2)
        self.assertEqual(event.limit_cycles_action, "expire")
        self.assertEqual(event.limit_cycles_expires_after, 1)
        self.assertEqual(event.limit_cycles_expires_type, "days")
        self.assertEqual(event.prorated_trial, 0)
        self.assertEqual(event.trial, 1)
        self.assertEqual(event.trial_days, 7)
        self.assertEqual(event.trial_amount, 0.00)
        self.assertEqual(event.trial_tax_amount, 0.00)
        self.assertEqual(event.trial_total, 0.00)
        self.assertEqual(event.status, "pending")
        self.assertEqual(event.created_at, datetime.strptime("2022-10-14 21:36:42", "%Y-%m-%d %H:%M:%S"))
        self.assertEqual(event.total, 9.00)
        self.assertEqual(event.tax_rate, 0.000)
        self.assertEqual(event.tax_amount, 0.00)
        self.assertEqual(event.tax_desc, "")
        self.assertEqual(event.tax_class, "standard")
        self.assertEqual(event.cc_last4, None)
        self.assertEqual(event.cc_exp_month, None)
        self.assertEqual(event.cc_exp_year, None)
        self.assertEqual(event.token, "")
        self.assertEqual(event.tax_compound, False)
        self.assertEqual(event.tax_shipping, True)
        self.assertEqual(event.response, None)

    def test_persist_data(self):
        def persist(event_str: str):
            data_dict = load_json(event_str)
            event = get_event(data_dict)

            MemberpressEventLog(
                sender="https://some-domain.com",
                username="mcdaniel",
                event=event.event,
                event_type=event.event_type,
                is_valid=event.is_valid,
                json=event.json,
            ).save()

        for file in os.listdir(EVENTS_FOLDER):
            if file.endswith(EXT):
                persist(event_str=file[:-5])



import logging

from rest_framework.views import APIView
from django.http import HttpResponse

from memberpress_client.decorators import app_logger
from memberpress_client.events import get_event
from memberpress_client.models import MemberpressEventLog
from memberpress_client.utils import get_user

logger = logging.getLogger(__name__)


class EventView(APIView):
    @app_logger
    def post(self, request):
        data = request.POST
        method = request.REQUEST_METHOD

        method = method
        event = get_event(data=data)
        try:
            username = event.username or request.user.username
        except Exception:
            username = "missing"

        MemberpressEventLog(
            sender=request.REMOTE_HOST,
            username=username,
            event=event.event,
            event_type=event.event_type,
            is_valid=event.is_valid,
            json=event.json,
        ).save()
        return HttpResponse(status=201)
